@@ -25,16 +25,17 @@
 
 #include <utils/savedaction.h>
 
-#include <utils/qtcassert.h>
-#include <utils/pathchooser.h>
-#include <utils/pathlisteditor.h>
-
-#include <QDebug>
-#include <QSettings>
+#include "pathchooser.h"
+#include "pathlisteditor.h"
+#include "qtcassert.h"
+#include "qtcsettings.h"
 
+#include <QActionGroup>
 #include <QCheckBox>
+#include <QDebug>
 #include <QGroupBox>
 #include <QLineEdit>
+#include <QSettings>
 #include <QSpinBox>
 #include <QTextEdit>
 
@@ -54,9 +55,9 @@ namespace Utils {
 */
 
 SavedAction::SavedAction(QObject *parent)
-  : QAction(parent)
 {
-    connect(this, &QAction::triggered, this, &SavedAction::actionTriggered);
+    setParent(parent);
+    connect(&m_action, &QAction::triggered, this, &SavedAction::actionTriggered);
 }
 
 
@@ -82,8 +83,8 @@ void SavedAction::setValue(const QVariant &value, bool doemit)
     if (value == m_value)
         return;
     m_value = value;
-    if (this->isCheckable())
-        this->setChecked(m_value.toBool());
+    if (m_action.isCheckable())
+        m_action.setChecked(m_value.toBool());
     if (doemit)
         emit valueChanged(m_value);
 }
@@ -113,66 +114,11 @@ void SavedAction::setDefaultValue(const QVariant &value)
 }
 
 
-/*!
-    Returns the key to be used when accessing the settings.
-
-    \sa settingsKey()
-*/
-QString SavedAction::settingsKey() const
-{
-    return m_settingsKey;
-}
-
-
-/*!
-    Sets the key to be used when accessing the settings.
-
-    \sa settingsKey()
-*/
-void SavedAction::setSettingsKey(const QString &key)
-{
-    m_settingsKey = key;
-}
-
-
-/*!
-    Sets the key and group to be used when accessing the settings.
-
-    \sa settingsKey()
-*/
-void SavedAction::setSettingsKey(const QString &group, const QString &key)
-{
-    m_settingsKey = key;
-    m_settingsGroup = group;
-}
-
-
-/*!
-    Sets the key to be used when accessing the settings.
-
-    \sa settingsKey()
-*/
-QString SavedAction::settingsGroup() const
-{
-    return m_settingsGroup;
-}
-
-/*!
-    Sets the group to be used when accessing the settings.
-
-    \sa settingsGroup()
-*/
-void SavedAction::setSettingsGroup(const QString &group)
-{
-    m_settingsGroup = group;
-}
-
 QString SavedAction::toString() const
 {
     return QLatin1String("value: ") + m_value.toString()
         + QLatin1String("  defaultvalue: ") + m_defaultValue.toString()
-        + QLatin1String("  settingskey: ") + m_settingsGroup
-        + QLatin1Char('/') + m_settingsKey;
+        + QLatin1String("  settingskey: ") + settingsKey();
 }
 
 /*
@@ -183,11 +129,11 @@ QString SavedAction::toString() const
 */
 void SavedAction::readSettings(const QSettings *settings)
 {
-    if (m_settingsGroup.isEmpty() || m_settingsKey.isEmpty())
+    if (settingsKey().isEmpty())
         return;
-    QVariant var = settings->value(m_settingsGroup + QLatin1Char('/') + m_settingsKey, m_defaultValue);
+    QVariant var = settings->value(settingsKey(), m_defaultValue);
     // work around old ini files containing @Invalid() entries
-    if (isCheckable() && !var.isValid())
+    if (m_action.isCheckable() && !var.isValid())
         var = false;
     setValue(var);
 }
@@ -200,11 +146,9 @@ void SavedAction::readSettings(const QSettings *settings)
 */
 void SavedAction::writeSettings(QSettings *settings)
 {
-    if (m_settingsGroup.isEmpty() || m_settingsKey.isEmpty())
+    if (settingsKey().isEmpty())
         return;
-    settings->beginGroup(m_settingsGroup);
-    settings->setValue(m_settingsKey, m_value);
-    settings->endGroup();
+    QtcSettings::setValueWithDefault(settings, settingsKey(), m_value, m_defaultValue);
 }
 
 /*
@@ -235,7 +179,7 @@ void SavedAction::connectWidget(QWidget *widget, ApplyMode applyMode)
     } else if (auto spinBox = qobject_cast<QSpinBox *>(widget)) {
         spinBox->setValue(m_value.toInt());
         if (applyMode == ImmediateApply) {
-            connect(spinBox, static_cast<void(QSpinBox::*)(int)>(&QSpinBox::valueChanged),
+            connect(spinBox, QOverload<int>::of(&QSpinBox::valueChanged),
                     this, [this, spinBox]() { setValue(spinBox->value()); });
         }
     } else if (auto lineEdit = qobject_cast<QLineEdit *>(widget)) {
@@ -274,7 +218,7 @@ void SavedAction::connectWidget(QWidget *widget, ApplyMode applyMode)
 
     // Copy tooltip, but only if there's nothing explcitly set on the widget yet.
     if (widget->toolTip().isEmpty())
-        widget->setToolTip(toolTip());
+        widget->setToolTip(m_action.toolTip());
 }
 
 /*
@@ -329,20 +273,26 @@ void SavedAction::setDialogText(const QString &dialogText)
 
 void SavedAction::actionTriggered(bool)
 {
-    if (isCheckable())
-        setValue(isChecked());
-    if (actionGroup() && actionGroup()->isExclusive()) {
+    if (m_action.isCheckable())
+        setValue(m_action.isChecked());
+    if (m_action.actionGroup() && m_action.actionGroup()->isExclusive()) {
         // FIXME: should be taken care of more directly
-        foreach (QAction *act, actionGroup()->actions())
+        const QList<QAction *> actions = m_action.actionGroup()->actions();
+        for (QAction *act : actions)
             if (auto dact = qobject_cast<SavedAction *>(act))
-                dact->setValue(bool(act == this));
+                dact->setValue(bool(act == &m_action));
     }
 }
 
+QAction *SavedAction::action()
+{
+    return &m_action;
+}
+
 void SavedAction::trigger(const QVariant &data)
 {
-    setData(data);
-    QAction::trigger();
+    m_action.setData(data);
+    m_action.trigger();
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -360,13 +310,13 @@ void SavedActionSet::insert(SavedAction *action, QWidget *widget)
 
 void SavedActionSet::apply(QSettings *settings)
 {
-    foreach (SavedAction *action, m_list)
+    for (SavedAction *action : qAsConst(m_list))
         action->apply(settings);
 }
 
 void SavedActionSet::finish()
 {
-    foreach (SavedAction *action, m_list)
+    for (SavedAction *action : qAsConst(m_list))
         action->disconnectWidget();
 }
 
