@@ -134,6 +134,16 @@ void SystemImage::setPlatform(SdkPlatform *platform)
     m_platform = platform;
 }
 
+int SystemImage::apiLevel() const
+{
+    return m_apiLevel;
+}
+
+void SystemImage::setApiLevel(const int apiLevel)
+{
+    m_apiLevel = apiLevel;
+}
+
 SdkPlatform::SdkPlatform(QVersionNumber version, QString sdkStylePathStr, int api, QObject *parent) :
     AndroidSdkPackage(version, sdkStylePathStr, parent),
     m_apiLevel(api)
@@ -144,7 +154,7 @@ SdkPlatform::SdkPlatform(QVersionNumber version, QString sdkStylePathStr, int ap
 
 SdkPlatform::~SdkPlatform()
 {
-    for (SystemImage *image : m_systemImages)
+    for (SystemImage *image : qAsConst(m_systemImages))
         delete image;
     m_systemImages.clear();
 }
@@ -284,4 +294,34 @@ AndroidSdkPackage::PackageType ExtraTools::type() const
     return AndroidSdkPackage::ExtraToolsPackage;
 }
 
+Ndk::Ndk(QVersionNumber revision, QString sdkStylePathStr, QObject *parent) :
+    AndroidSdkPackage(revision, sdkStylePathStr, parent)
+{
+}
+
+bool Ndk::isValid() const
+{
+    return installedLocation().exists();
+}
+
+AndroidSdkPackage::PackageType Ndk::type() const
+{
+    return AndroidSdkPackage::NDKPackage;
+}
+
+GenericSdkPackage::GenericSdkPackage(QVersionNumber revision, QString sdkStylePathStr, QObject  *parent) :
+    AndroidSdkPackage(revision, sdkStylePathStr, parent)
+{
+}
+
+bool GenericSdkPackage::isValid() const
+{
+    return installedLocation().exists();
+}
+
+AndroidSdkPackage::PackageType GenericSdkPackage::type() const
+{
+    return AndroidSdkPackage::GenericSdkPackage;
+}
+
 } // namespace Android
