@@ -33,16 +33,54 @@
 
 #include <utils/qtcassert.h>
 
-#include <QSysInfo>
 #include <QApplication>
+#include <QDebug>
 #include <QStandardPaths>
+#include <QSysInfo>
 
 /*!
     \namespace Core
+    \inmodule QtCreator
     \brief The Core namespace contains all classes that make up the Core plugin
     which constitute the basic functionality of \QC.
 */
 
+/*!
+    \enum Core::FindFlag
+    This enum holds the find flags.
+
+    \value FindBackward
+           Searches backwards.
+    \value FindCaseSensitively
+           Considers case when searching.
+    \value FindWholeWords
+           Finds only whole words.
+    \value FindRegularExpression
+           Uses a regular epression as a search term.
+    \value FindPreserveCase
+           Preserves the case when replacing search terms.
+*/
+
+/*!
+    \enum Core::ICore::ContextPriority
+
+    This enum defines the priority of additional contexts.
+
+    \value High
+           Additional contexts that have higher priority than contexts from
+           Core::IContext instances.
+    \value Low
+           Additional contexts that have lower priority than contexts from
+           Core::IContext instances.
+
+    \sa Core::ICore::updateAdditionalContexts()
+*/
+
+/*!
+    \enum Core::SaveSettingsReason
+    \internal
+*/
+
 /*!
     \namespace Core::Internal
     \internal
@@ -50,219 +88,40 @@
 
 /*!
     \class Core::ICore
+    \inheaderfile coreplugin/icore.h
+    \inmodule QtCreator
+    \ingroup mainclasses
+
     \brief The ICore class allows access to the different parts that make up
     the basic functionality of \QC.
 
     You should never create a subclass of this interface. The one and only
     instance is created by the Core plugin. You can access this instance
-    from your plugin through \c{Core::instance()}.
-
-    \mainclass
+    from your plugin through instance().
 */
 
 /*!
-    \fn void ICore::showNewItemDialog(const QString &title,
-                                      const QList<IWizard *> &wizards,
-                                      const QString &defaultLocation = QString(),
-                                      const QVariantMap &extraVariables = QVariantMap())
+    \fn void Core::ICore::coreAboutToOpen()
 
-    Opens a dialog where the user can choose from a set of \a wizards that
-    create new files, classes, or projects.
-
-    The \a title argument is shown as the dialog title. The path where the
-    files will be created (if the user does not change it) is set
-    in \a defaultLocation. It defaults to the path of the file manager's
-    current file.
-
-    \sa Core::DocumentManager
+    Indicates that all plugins have been loaded and the main window is about to
+    be shown.
 */
 
 /*!
-    \fn bool ICore::showOptionsDialog(Id group, Id page, QWidget *parent = 0);
-
-    Opens the application \gui Options (or \gui Preferences) dialog with preselected
-    \a page in the specified \a group.
-
-    The arguments refer to the string IDs of the corresponding IOptionsPage.
-*/
-
-/*!
-    \fn bool ICore::showWarningWithOptions(const QString &title, const QString &text,
-                                   const QString &details = QString(),
-                                   Id settingsCategory = Id(),
-                                   Id settingsId = Id(),
-                                   QWidget *parent = 0)
-
-    Shows a warning message with a button that opens a settings page.
-
-    Should be used to display configuration errors and point users to the setting.
-    Returns \c true if the settings dialog was accepted.
-*/
-
-
-/*!
-    \fn QSettings *ICore::settings(QSettings::Scope scope = QSettings::UserScope)
-
-    Returns the application's main settings object.
-
-    You can use it to retrieve or set application wide settings
-    (in contrast to session or project specific settings).
-
-    If \a scope is \c QSettings::UserScope (the default), the
-    users settings will be read from the users settings, with
-    a fallback to global settings provided with \QC.
-
-    If \a scope is \c QSettings::SystemScope, only the system settings
-    shipped with the current version of \QC will be read. This
-    functionality exists for internal purposes only.
-
-    \see settingsDatabase()
-*/
-
-/*!
-    \fn SettingsDatabase *ICore::settingsDatabase()
-
-    Returns the application's settings database.
-
-    The settings database is meant as an alternative to the regular settings
-    object. It is more suitable for storing large amounts of data. The settings
-    are application wide.
-
-    \see SettingsDatabase
-*/
-
-/*!
-    \fn QPrinter *ICore::printer()
-
-    Returns the application's printer object.
-
-    Always use this printer object for printing, so the different parts of the
-    application re-use its settings.
-*/
-
-/*!
-    \fn QString ICore::resourcePath()
-
-    Returns the absolute path that is used for resources like
-    project templates and the debugger macros.
-
-    This abstraction is needed to avoid platform-specific code all over
-    the place, since on Mac OS X, for example, the resources are part of the
-    application bundle.
-*/
-
-
-/*!
-    \fn QString ICore::userResourcePath()
-
-    Returns the absolute path in the users directory that is used for
-    resources like project templates.
-
-    Use this function for finding the place for resources that the user may
-    write to, for example, to allow for custom palettes or templates.
-*/
-
-/*!
-    \fn QWidget *ICore::mainWindow()
-
-    Returns the main application window.
-
-    For dialog parents use \c dialogParent().
-*/
-
-/*!
-    \fn QWidget *ICore::dialogParent()
-
-    Returns a widget pointer suitable to use as parent for QDialogs.
- */
-
-/*!
-    \fn IContext *ICore::currentContextObject()
-
-    Returns the context object of the current main context.
-
-    \sa ICore::updateAdditionalContexts()
-    \sa ICore::addContextObject()
-*/
-
-/*!
-    \fn void ICore::updateAdditionalContexts(const Context &remove, const Context &add)
-    Changes the currently active additional contexts.
-
-    Removes the list of additional contexts specified by \a remove and adds the
-    list of additional contexts specified by \a add.
-
-    \sa ICore::hasContext()
-*/
-
-/*!
-    \fn bool ICore::hasContext(int context) const
-    Returns whether the given \a context is currently one of the active contexts.
-
-    \sa ICore::updateAdditionalContexts()
-    \sa ICore::addContextObject()
-*/
-
-/*!
-    \fn void ICore::addContextObject(IContext *context)
-    Registers an additional \a context object.
-
-    After registration this context object gets automatically the
-    current context object whenever its widget gets focus.
-
-    \sa ICore::removeContextObject()
-    \sa ICore::updateAdditionalContexts()
-    \sa ICore::currentContextObject()
-*/
-
-/*!
-    \fn void ICore::removeContextObject(IContext *context)
-    Unregisters a \a context object from the list of know contexts.
-
-    \sa ICore::addContextObject()
-    \sa ICore::updateAdditionalContexts()
-    \sa ICore::currentContextObject()
-}
-*/
-
-/*!
-    \fn void ICore::openFiles(const QStringList &fileNames, OpenFilesFlags flags = None)
-    Opens all files from a list of \a fileNames like it would be
-    done if they were given to \QC on the command line, or
-    they were opened via \gui File > \gui Open.
-*/
-
-/*!
-    \fn ICore::ICore(Internal::MainWindow *mw)
-    \internal
-*/
-
-/*!
-    \fn ICore::~ICore()
-    \internal
-*/
-
-/*!
-    \fn void ICore::coreOpened()
+    \fn void Core::ICore::coreOpened()
     Indicates that all plugins have been loaded and the main window is shown.
 */
 
 /*!
-    \fn void ICore::saveSettingsRequested()
+    \fn void Core::ICore::saveSettingsRequested(Core::ICore::SaveSettingsReason reason)
     Signals that the user has requested that the global settings
-    should be saved to disk.
+    should be saved to disk for a \a reason.
 
-    At the moment that happens when the application is closed, and on \gui{Save All}.
+    At the moment that happens when the application is closed, and on \uicontrol{Save All}.
 */
 
 /*!
-    \fn void ICore::optionsDialogRequested()
-    Enables plugins to perform actions just before the \gui Tools > \gui Options
-    dialog is shown.
-*/
-
-/*!
-    \fn void ICore::coreAboutToClose()
+    \fn void Core::ICore::coreAboutToClose()
     Enables plugins to perform some pre-end-of-life actions.
 
     The application is guaranteed to shut down after this signal is emitted.
@@ -271,13 +130,13 @@
 */
 
 /*!
-    \fn void ICore::contextAboutToChange(const QList<Core::IContext *> &context)
+    \fn void Core::ICore::contextAboutToChange(const QList<Core::IContext *> &context)
     Indicates that a new \a context will shortly become the current context
     (meaning that its widget got focus).
 */
 
 /*!
-    \fn void ICore::contextChanged(const Core::Context &context)
+    \fn void Core::ICore::contextChanged(const Core::Context &context)
     Indicates that a new \a context just became the current context. This includes the context
     from the focus object as well as the additional context.
 */
@@ -295,10 +154,10 @@
 #include <QMessageBox>
 #include <QPushButton>
 #include <QStatusBar>
-#include <QTimer>
 
 using namespace Core::Internal;
 using namespace ExtensionSystem;
+using namespace Utils;
 
 namespace Core {
 
@@ -306,16 +165,28 @@ namespace Core {
 static ICore *m_instance = nullptr;
 static MainWindow *m_mainwindow = nullptr;
 
+/*!
+    Returns the pointer to the instance. Only use for connecting to signals.
+*/
 ICore *ICore::instance()
 {
     return m_instance;
 }
 
+/*!
+    Returns whether the new item dialog is currently open.
+*/
 bool ICore::isNewItemDialogRunning()
 {
     return NewDialog::currentDialog() || IWizardFactory::isWizardRunning();
 }
 
+/*!
+    Returns the currently open new item dialog widget, or \c nullptr if there is none.
+
+    \sa isNewItemDialogRunning()
+    \sa showNewItemDialog()
+*/
 QWidget *ICore::newItemDialog()
 {
     if (NewDialog::currentDialog())
@@ -323,25 +194,49 @@ QWidget *ICore::newItemDialog()
     return IWizardFactory::currentWizard();
 }
 
+/*!
+    \internal
+*/
 ICore::ICore(MainWindow *mainwindow)
 {
     m_instance = this;
     m_mainwindow = mainwindow;
     // Save settings once after all plugins are initialized:
     connect(PluginManager::instance(), &PluginManager::initializationDone,
-            this, &ICore::saveSettings);
+            this, [] { ICore::saveSettings(ICore::InitializationDone); });
     connect(PluginManager::instance(), &PluginManager::testsFinished, [this] (int failedTests) {
         emit coreAboutToClose();
+        if (failedTests != 0)
+            qWarning("Test run was not successful: %d test(s) failed.", failedTests);
         QCoreApplication::exit(failedTests);
     });
 }
 
+/*!
+    \internal
+*/
 ICore::~ICore()
 {
     m_instance = nullptr;
     m_mainwindow = nullptr;
 }
 
+/*!
+    Opens a dialog where the user can choose from a set of \a factories that
+    create new files or projects.
+
+    The \a title argument is shown as the dialog title. The path where the
+    files will be created (if the user does not change it) is set
+    in \a defaultLocation. Defaults to DocumentManager::projectsDirectory()
+    or DocumentManager::fileDialogLastVisitedDirectory(), depending on wizard
+    kind.
+
+    Additional variables for the wizards are set in \a extraVariables.
+
+    \sa Core::DocumentManager
+    \sa isNewItemDialogRunning()
+    \sa newItemDialog()
+*/
 void ICore::showNewItemDialog(const QString &title,
                               const QList<IWizardFactory *> &factories,
                               const QString &defaultLocation,
@@ -357,16 +252,38 @@ void ICore::showNewItemDialog(const QString &title,
     updateNewItemDialogState();
 }
 
+/*!
+    Opens the options dialog on the specified \a page. The dialog's \a parent
+    defaults to dialogParent(). If the dialog is already shown when this method
+    is called, it is just switched to the specified \a page.
+
+    Returns whether the user accepted the dialog.
+
+    \sa msgShowOptionsDialog()
+    \sa msgShowOptionsDialogToolTip()
+*/
 bool ICore::showOptionsDialog(const Id page, QWidget *parent)
 {
     return executeSettingsDialog(parent ? parent : dialogParent(), page);
 }
 
+/*!
+    Returns the text to use on buttons that open the options dialog.
+
+    \sa showOptionsDialog()
+    \sa msgShowOptionsDialogToolTip()
+*/
 QString ICore::msgShowOptionsDialog()
 {
     return QCoreApplication::translate("Core", "Configure...", "msgShowOptionsDialog");
 }
 
+/*!
+    Returns the tool tip to use on buttons that open the options dialog.
+
+    \sa showOptionsDialog()
+    \sa msgShowOptionsDialog()
+*/
 QString ICore::msgShowOptionsDialogToolTip()
 {
     if (Utils::HostOsInfo::isMacHost())
@@ -377,8 +294,20 @@ QString ICore::msgShowOptionsDialogToolTip()
                                            "msgShowOptionsDialogToolTip (non-mac version)");
 }
 
-// Display a warning with an additional button to open
-// the settings dialog at a specified page if settingsId is nonempty.
+/*!
+    Creates a message box with \a parent that contains a \uicontrol Configure
+    button for opening the settings page specified by \a settingsId.
+
+    The dialog has \a title and displays the message \a text and detailed
+    information specified by \a details.
+
+    Use this function to display configuration errors and to point users to the
+    setting they should fix.
+
+    Returns \c true if the user accepted the settings dialog.
+
+    \sa showOptionsDialog()
+*/
 bool ICore::showWarningWithOptions(const QString &title, const QString &text,
                                    const QString &details, Id settingsId, QWidget *parent)
 {
@@ -390,14 +319,30 @@ bool ICore::showWarningWithOptions(const QString &title, const QString &text,
         msgBox.setDetailedText(details);
     QAbstractButton *settingsButton = nullptr;
     if (settingsId.isValid())
-        settingsButton = msgBox.addButton(tr("Settings..."), QMessageBox::AcceptRole);
+        settingsButton = msgBox.addButton(msgShowOptionsDialog(), QMessageBox::AcceptRole);
     msgBox.exec();
     if (settingsButton && msgBox.clickedButton() == settingsButton)
         return showOptionsDialog(settingsId);
     return false;
 }
 
-QSettings *ICore::settings(QSettings::Scope scope)
+/*!
+    Returns the application's main settings object.
+
+    You can use it to retrieve or set application-wide settings
+    (in contrast to session or project specific settings).
+
+    If \a scope is \c QSettings::UserScope (the default), the
+    settings will be read from the user's settings, with
+    a fallback to global settings provided with \QC.
+
+    If \a scope is \c QSettings::SystemScope, only the installation settings
+    shipped with the current version of \QC will be read. This
+    functionality exists for internal purposes only.
+
+    \sa settingsDatabase()
+*/
+QtcSettings *ICore::settings(QSettings::Scope scope)
 {
     if (scope == QSettings::UserScope)
         return PluginManager::settings();
@@ -405,26 +350,67 @@ QSettings *ICore::settings(QSettings::Scope scope)
         return PluginManager::globalSettings();
 }
 
+/*!
+    Returns the application's settings database.
+
+    The settings database is meant as an alternative to the regular settings
+    object. It is more suitable for storing large amounts of data. The settings
+    are application wide.
+
+    \sa SettingsDatabase
+    \sa settings()
+*/
 SettingsDatabase *ICore::settingsDatabase()
 {
     return m_mainwindow->settingsDatabase();
 }
 
+/*!
+    Returns the application's printer object.
+
+    Always use this printer object for printing, so the different parts of the
+    application re-use its settings.
+*/
 QPrinter *ICore::printer()
 {
     return m_mainwindow->printer();
 }
 
+/*!
+    Returns the locale string for the user interface language that is currently
+    configured in \QC. Use this to install your plugin's translation file with
+    QTranslator.
+*/
 QString ICore::userInterfaceLanguage()
 {
     return qApp->property("qtc_locale").toString();
 }
 
+/*!
+    Returns the absolute path that is used for resources like
+    project templates and the debugger macros.
+
+    This abstraction is needed to avoid platform-specific code all over
+    the place, since on \macos, for example, the resources are part of the
+    application bundle.
+
+    \sa userResourcePath()
+*/
 QString ICore::resourcePath()
 {
     return QDir::cleanPath(QCoreApplication::applicationDirPath() + '/' + RELATIVE_DATA_PATH);
 }
 
+/*!
+    Returns the absolute path in the users directory that is used for
+    resources like project templates.
+
+    Use this function for finding the place for resources that the user may
+    write to, for example, to allow for custom palettes or templates.
+
+    \sa resourcePath()
+*/
+
 QString ICore::userResourcePath()
 {
     // Create qtcreator dir if it doesn't yet exist
@@ -440,48 +426,127 @@ QString ICore::userResourcePath()
     return urp;
 }
 
+/*!
+    Returns a writable path that can be used for persistent cache files.
+*/
 QString ICore::cacheResourcePath()
 {
     return QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
 }
 
+/*!
+    Returns the path to resources written by the installer, for example
+    pre-defined kits and toolchains.
+*/
 QString ICore::installerResourcePath()
 {
-    return QFileInfo(settings(QSettings::SystemScope)->fileName()).path() + '/'
-           + Constants::IDE_ID;
+    return QFileInfo(settings(QSettings::SystemScope)->fileName()).path() + '/' + Constants::IDE_ID;
 }
 
 /*!
-    Returns the path to the command line tools that are shipped with \QC (corresponding
-    to the IDE_LIBEXEC_PATH qmake variable).
+    Returns the path to the plugins that are included in the \QC installation.
+
+    \internal
+*/
+QString ICore::pluginPath()
+{
+    return QDir::cleanPath(QCoreApplication::applicationDirPath() + '/' + RELATIVE_PLUGIN_PATH);
+}
+
+/*!
+    Returns the path where user-specific plugins should be written.
+
+    \internal
+*/
+QString ICore::userPluginPath()
+{
+    QString pluginPath = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
+    if (Utils::HostOsInfo::isAnyUnixHost() && !Utils::HostOsInfo::isMacHost())
+        pluginPath += "/data";
+    pluginPath += '/' + QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR) + '/';
+    pluginPath += QLatin1String(Utils::HostOsInfo::isMacHost() ? Core::Constants::IDE_DISPLAY_NAME
+                                                               : Core::Constants::IDE_ID);
+    pluginPath += "/plugins/";
+    pluginPath += QString::number(IDE_VERSION_MAJOR) + '.' + QString::number(IDE_VERSION_MINOR)
+                  + '.' + QString::number(IDE_VERSION_RELEASE);
+    return pluginPath;
+}
+
+/*!
+    Returns the path to the command line tools that are included in the \QC
+    installation.
  */
 QString ICore::libexecPath()
 {
     return QDir::cleanPath(QApplication::applicationDirPath() + '/' + RELATIVE_LIBEXEC_PATH);
 }
 
+QString ICore::crashReportsPath()
+{
+    if (Utils::HostOsInfo::isMacHost())
+        return libexecPath() + "/crashpad_reports/completed";
+    else
+        return libexecPath() + "/crashpad_reports/reports";
+}
+
+QString ICore::ideDisplayName()
+{
+    return Constants::IDE_DISPLAY_NAME;
+}
+
 static QString clangIncludePath(const QString &clangVersion)
 {
     return "/lib/clang/" + clangVersion + "/include";
 }
 
-QString ICore::clangIncludeDirectory(const QString &clangVersion, const QString &clangResourceDirectory)
+/*!
+    \internal
+*/
+QString ICore::clangIncludeDirectory(const QString &clangVersion,
+                                     const QString &clangFallbackIncludeDir)
 {
     QDir dir(libexecPath() + "/clang" + clangIncludePath(clangVersion));
     if (!dir.exists() || !QFileInfo(dir, "stdint.h").exists())
-        dir = QDir(clangResourceDirectory);
+        dir = QDir(clangFallbackIncludeDir);
     return QDir::toNativeSeparators(dir.canonicalPath());
 }
 
-QString ICore::clangExecutable(const QString &clangBinDirectory)
+/*!
+    \internal
+*/
+static QString clangBinary(const QString &binaryBaseName, const QString &clangBinDirectory)
 {
     const QString hostExeSuffix(QTC_HOST_EXE_SUFFIX);
-    QFileInfo executable(libexecPath() + "/clang/bin/clang" + hostExeSuffix);
+    QFileInfo executable(ICore::libexecPath() + "/clang/bin/" + binaryBaseName + hostExeSuffix);
     if (!executable.exists())
-        executable = QFileInfo(clangBinDirectory + "/clang" + hostExeSuffix);
+        executable = QFileInfo(clangBinDirectory + "/" + binaryBaseName + hostExeSuffix);
     return QDir::toNativeSeparators(executable.canonicalFilePath());
 }
 
+/*!
+    \internal
+*/
+QString ICore::clangExecutable(const QString &clangBinDirectory)
+{
+    return clangBinary("clang", clangBinDirectory);
+}
+
+/*!
+    \internal
+*/
+QString ICore::clangTidyExecutable(const QString &clangBinDirectory)
+{
+    return clangBinary("clang-tidy", clangBinDirectory);
+}
+
+/*!
+    \internal
+*/
+QString ICore::clazyStandaloneExecutable(const QString &clangBinDirectory)
+{
+    return clangBinary("clazy-standalone", clangBinDirectory);
+}
+
 static QString compilerString()
 {
 #if defined(Q_CC_CLANG) // must be before GNU, because clang claims to be GNU too
@@ -496,6 +561,8 @@ static QString compilerString()
 #elif defined(Q_CC_MSVC)
     if (_MSC_VER > 1999)
         return QLatin1String("MSVC <unknown>");
+    if (_MSC_VER >= 1920)
+        return QLatin1String("MSVC 2019");
     if (_MSC_VER >= 1910)
         return QLatin1String("MSVC 2017");
     if (_MSC_VER >= 1900)
@@ -504,6 +571,10 @@ static QString compilerString()
     return QLatin1String("<unknown compiler>");
 }
 
+/*!
+    Returns a string with the IDE's name and version, in the form "\QC X.Y.Z".
+    Use this for "Generated by" strings and similar tasks.
+*/
 QString ICore::versionString()
 {
     QString ideVersionDescription;
@@ -514,6 +585,9 @@ QString ICore::versionString()
                              ideVersionDescription);
 }
 
+/*!
+    \internal
+*/
 QString ICore::buildCompatibilityString()
 {
     return tr("Based on Qt %1 (%2, %3 bit)").arg(QLatin1String(qVersion()),
@@ -521,43 +595,87 @@ QString ICore::buildCompatibilityString()
                                                  QString::number(QSysInfo::WordSize));
 }
 
+/*!
+    Returns the top level IContext of the current context, or \c nullptr if
+    there is none.
+
+    \sa updateAdditionalContexts()
+    \sa addContextObject()
+    \sa {The Action Manager and Commands}
+*/
 IContext *ICore::currentContextObject()
 {
     return m_mainwindow->currentContextObject();
 }
 
+/*!
+    Returns the widget of the top level IContext of the current context, or \c
+    nullptr if there is none.
+
+    \sa currentContextObject()
+*/
 QWidget *ICore::currentContextWidget()
 {
     IContext *context = currentContextObject();
     return context ? context->widget() : nullptr;
 }
 
+/*!
+    Returns the registered IContext instance for the specified \a widget,
+    if any.
+*/
 IContext *ICore::contextObject(QWidget *widget)
 {
     return m_mainwindow->contextObject(widget);
 }
 
+/*!
+    Returns the main window of the application.
 
+    For dialog parents use dialogParent().
+
+    \sa dialogParent()
+*/
 QMainWindow *ICore::mainWindow()
 {
     return m_mainwindow;
 }
 
+/*!
+    Returns a widget pointer suitable to use as parent for QDialogs.
+*/
 QWidget *ICore::dialogParent()
 {
     QWidget *active = QApplication::activeModalWidget();
     if (!active)
         active = QApplication::activeWindow();
-    if (!active)
+    if (!active || (active && active->windowFlags().testFlag(Qt::SplashScreen)))
         active = m_mainwindow;
     return active;
 }
 
+/*!
+    \internal
+*/
 QStatusBar *ICore::statusBar()
 {
     return m_mainwindow->statusBar();
 }
 
+/*!
+    Returns a central InfoBar that is shown in \QC's main window.
+    Use for notifying the user of something without interrupting with
+    dialog. Use sparingly.
+*/
+Utils::InfoBar *ICore::infoBar()
+{
+    return m_mainwindow->infoBar();
+}
+
+/*!
+    Raises and activates the window for \a widget. This contains workarounds
+    for X11.
+*/
 void ICore::raiseWindow(QWidget *widget)
 {
     if (!widget)
@@ -571,61 +689,120 @@ void ICore::raiseWindow(QWidget *widget)
     }
 }
 
+/*!
+    Removes the contexts specified by \a remove from the list of active
+    additional contexts, and adds the contexts specified by \a add with \a
+    priority.
+
+    The additional contexts are not associated with an IContext instance.
+
+    High priority additional contexts have higher priority than the contexts
+    added by IContext instances, low priority additional contexts have lower
+    priority than the contexts added by IContext instances.
+
+    \sa addContextObject()
+    \sa {The Action Manager and Commands}
+*/
 void ICore::updateAdditionalContexts(const Context &remove, const Context &add,
                                      ContextPriority priority)
 {
     m_mainwindow->updateAdditionalContexts(remove, add, priority);
 }
 
+/*!
+    Adds \a context with \a priority to the list of active additional contexts.
+
+    \sa updateAdditionalContexts()
+*/
 void ICore::addAdditionalContext(const Context &context, ContextPriority priority)
 {
     m_mainwindow->updateAdditionalContexts(Context(), context, priority);
 }
 
+/*!
+    Removes \a context from the list of active additional contexts.
+
+    \sa updateAdditionalContexts()
+*/
 void ICore::removeAdditionalContext(const Context &context)
 {
     m_mainwindow->updateAdditionalContexts(context, Context(), ContextPriority::Low);
 }
 
+/*!
+    Adds \a context to the list of registered IContext instances.
+    Whenever the IContext's \l{IContext::widget()}{widget} is in the application
+    focus widget's parent hierarchy, its \l{IContext::context()}{context} is
+    added to the list of active contexts.
+
+    \sa removeContextObject()
+    \sa updateAdditionalContexts()
+    \sa currentContextObject()
+    \sa {The Action Manager and Commands}
+*/
 void ICore::addContextObject(IContext *context)
 {
     m_mainwindow->addContextObject(context);
 }
 
+/*!
+    Unregisters a \a context object from the list of registered IContext
+    instances. IContext instances are automatically removed when they are
+    deleted.
+
+    \sa addContextObject()
+    \sa updateAdditionalContexts()
+    \sa currentContextObject()
+*/
 void ICore::removeContextObject(IContext *context)
 {
     m_mainwindow->removeContextObject(context);
 }
 
+/*!
+    Registers a \a window with the specified \a context. Registered windows are
+    shown in the \uicontrol Window menu and get registered for the various
+    window related actions, like the minimize, zoom, fullscreen and close
+    actions.
+
+    Whenever the application focus is in \a window, its \a context is made
+    active.
+*/
 void ICore::registerWindow(QWidget *window, const Context &context)
 {
     new WindowSupport(window, context); // deletes itself when widget is destroyed
 }
 
+/*!
+    Opens files using \a arguments and \a flags like it would be
+    done if they were given to \QC on the command line, or
+    they were opened via \uicontrol File > \uicontrol Open.
+*/
+
 void ICore::openFiles(const QStringList &arguments, ICore::OpenFilesFlags flags)
 {
-    m_mainwindow->openFiles(arguments, flags);
+    MainWindow::openFiles(arguments, flags);
 }
 
 /*!
-    \fn ICore::addCloseCoreListener
+    Provides a hook for plugins to veto on closing the application.
 
-    \brief The \c ICore::addCloseCoreListener function provides a hook for plugins
-    to veto on closing the application.
-
-    When the application window requests a close, all listeners are called.
-    If one if these calls returns \c false, the process is aborted and the
-    event is ignored. If all calls return \c true, \c ICore::coreAboutToClose()
-    is emitted and the event is accepted or performed..
+    When the application window requests a close, all listeners are called. If
+    one of the \a listener calls returns \c false, the process is aborted and
+    the event is ignored. If all calls return \c true, coreAboutToClose()
+    is emitted and the event is accepted or performed.
 */
 void ICore::addPreCloseListener(const std::function<bool ()> &listener)
 {
     m_mainwindow->addPreCloseListener(listener);
 }
 
+/*!
+    \internal
+*/
 QString ICore::systemInformation()
 {
-    QString result = PluginManager::instance()->systemInformation() + '\n';
+    QString result = PluginManager::systemInformation() + '\n';
     result += versionString() + '\n';
     result += buildCompatibilityString() + '\n';
 #ifdef IDE_REVISION
@@ -656,7 +833,7 @@ public:
     {
         QTC_ASSERT(watched == m_widget, return false);
         if (event->type() == QEvent::Show)
-            QTimer::singleShot(0, this, &ScreenShooter::helper);
+            QMetaObject::invokeMethod(this, &ScreenShooter::helper, Qt::QueuedConnection);
         return false;
     }
 
@@ -681,26 +858,46 @@ public:
     QRect m_rc;
 };
 
+/*!
+    \internal
+*/
 void ICore::setupScreenShooter(const QString &name, QWidget *w, const QRect &rc)
 {
     if (!screenShotsPath().isEmpty())
         new ScreenShooter(w, name, rc);
 }
 
-void ICore::saveSettings()
+/*!
+    Restarts \QC and restores the last session.
+*/
+void ICore::restart()
 {
-    emit m_instance->saveSettingsRequested();
+    m_mainwindow->restart();
+}
+
+/*!
+    \internal
+*/
+void ICore::saveSettings(SaveSettingsReason reason)
+{
+    emit m_instance->saveSettingsRequested(reason);
     m_mainwindow->saveSettings();
 
     ICore::settings(QSettings::SystemScope)->sync();
     ICore::settings(QSettings::UserScope)->sync();
 }
 
+/*!
+    \internal
+*/
 QStringList ICore::additionalAboutInformation()
 {
     return m_mainwindow->additionalAboutInformation();
 }
 
+/*!
+    \internal
+*/
 void ICore::appendAboutInformation(const QString &line)
 {
     m_mainwindow->appendAboutInformation(line);
