@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,34 +9,33 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "cpppointerdeclarationformatter.h"
 #include "cpptoolsplugin.h"
-#include "cpptoolsplugin.h"
 #include "cpptoolstestcase.h"
 
-#include <texteditor/plaintexteditor.h>
+#include <coreplugin/coreconstants.h>
+
+#include <texteditor/texteditor.h>
+#include <texteditor/textdocument.h>
+#include <texteditor/plaintexteditorfactory.h>
 
 #include <utils/fileutils.h>
 
+#include <cplusplus/Overview.h>
 #include <cplusplus/pp.h>
 
 #include <QDebug>
@@ -64,7 +63,7 @@ QString stripCursor(const QString &source)
     return copy;
 }
 
-class PointerDeclarationFormatterTestCase : public CppTools::Tests::TestCase
+class PointerDeclarationFormatterTestCase : public Tests::TestCase
 {
 public:
     PointerDeclarationFormatterTestCase(const QByteArray &source,
@@ -84,15 +83,18 @@ public:
         }
 
         // Write source to temprorary file
-        const QString filePath = QDir::tempPath() + QLatin1String("/file.h");
-        Document::Ptr document = Document::create(filePath);
-        QVERIFY(writeFile(document->fileName(), sourceWithoutCursorMarker.toUtf8()));
+        Tests::TemporaryDir temporaryDir;
+        QVERIFY(temporaryDir.isValid());
+        const QString filePath = temporaryDir.createFile("file.h",
+                                                         sourceWithoutCursorMarker.toUtf8());
+        QVERIFY(!filePath.isEmpty());
 
         // Preprocess source
         Environment env;
-        Preprocessor preprocess(0, &env);
+        Preprocessor preprocess(nullptr, &env);
         const QByteArray preprocessedSource = preprocess.run(filePath, sourceWithoutCursorMarker);
 
+        Document::Ptr document = Document::create(filePath);
         document->setUtf8Source(preprocessedSource);
         document->parse(parseMode);
         document->check();
@@ -101,37 +103,37 @@ public:
         QVERIFY(ast);
 
         // Open file
-        QScopedPointer<TextEditor::BaseTextEditorWidget> editorWidget(
-            new TextEditor::PlainTextEditorWidget);
+        QScopedPointer<TextEditor::BaseTextEditor> editor(
+                    TextEditor::PlainTextEditorFactory::createPlainTextEditor());
         QString error;
-        editorWidget->open(&error, document->fileName(), document->fileName());
+        editor->document()->open(&error, document->fileName(), document->fileName());
         QVERIFY(error.isEmpty());
 
         // Set cursor position
-        QTextCursor cursor = editorWidget->textCursor();
+        QTextCursor cursor = editor->textCursor();
         cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::MoveAnchor, cursorPosition);
-        editorWidget->setTextCursor(cursor);
+        editor->setTextCursor(cursor);
 
-        QTextDocument *textDocument = editorWidget->document();
+        QTextDocument *qtextDocument = editor->textDocument()->document();
         CppRefactoringFilePtr cppRefactoringFile
-            = CppRefactoringChanges::file(editorWidget.data(), document);
+            = CppRefactoringChanges::file(editor->editorWidget(), document);
 
         // Prepare for formatting
         Overview overview;
         overview.showReturnTypes = true;
         overview.showArgumentNames = true;
-        overview.starBindFlags = Overview::StarBindFlags(0);
+        overview.starBindFlags = Overview::StarBindFlags();
 
         // Run the formatter
         PointerDeclarationFormatter formatter(cppRefactoringFile, overview, cursorHandling);
         Utils::ChangeSet change = formatter.format(ast); // ChangeSet may be empty.
 
         // Apply change
-        QTextCursor changeCursor(textDocument);
+        QTextCursor changeCursor(qtextDocument);
         change.apply(&changeCursor);
 
         // Compare
-        QCOMPARE(textDocument->toPlainText(), expectedSource);
+        QCOMPARE(qtextDocument->toPlainText(), expectedSource);
     }
 };
 
@@ -488,7 +490,7 @@ void CppToolsPlugin::test_format_pointerdeclaration_multiple_declarators_data()
 
     QTest::newRow("function-pointer_in-start")
         << "char *s, @*(*foo)(char *s) = 0;"
-        << "char *s, *(*foo)(char * s) = 0;";
+        << "char *s, * (*foo)(char * s) = 0;";
 }
 
 void CppToolsPlugin::test_format_pointerdeclaration_multiple_matches()
@@ -532,8 +534,8 @@ void CppToolsPlugin::test_format_pointerdeclaration_multiple_matches_data()
         << "int g() { for (char * s; char * t = 0; s++) { char * u = 0; } }";
 
     QTest::newRow("multiple-declarators")
-        << "const char c, *s, *(*foo)(char *s) = 0;"
-        << "const char c, * s, *(*foo)(char * s) = 0;";
+        << "const char c, *s, * (*foo)(char *s) = 0;"
+        << "const char c, * s, * (*foo)(char * s) = 0;";
 
     QTest::newRow("complex")
         <<
@@ -566,7 +568,7 @@ void CppToolsPlugin::test_format_pointerdeclaration_multiple_matches_data()
            "    int (*bar)[] = 0;\n"
            "\n"
            "    char * s = 0, * f(int i) = 0;\n"
-           "    const char c, * s, *(*foo)(char * s) = 0;"
+           "    const char c, * s, * (*foo)(char * s) = 0;"
            "\n"
            "    for (char * s; char * t = 0; s++) { char * u = 0; }"
            "\n"
@@ -652,7 +654,7 @@ void CppToolsPlugin::test_format_pointerdeclaration_macros_data()
            "public:\n"
            "    typedef QPair<QString, QString> Item;\n"
            " \n"
-           "    Core::Id dataId(); // the higher the closer to top.\n"
+           "    Utils::Id dataId(); // the higher the closer to top.\n"
            " \n"
            "    unsigned int priority() = 0;\n"
            " \n"
@@ -673,7 +675,7 @@ void CppToolsPlugin::test_format_pointerdeclaration_macros_data()
            "public:\n"
            "    typedef QPair<QString, QString> Item;\n"
            " \n"
-           "    Core::Id dataId(); // the higher the closer to top.\n"
+           "    Utils::Id dataId(); // the higher the closer to top.\n"
            " \n"
            "    unsigned int priority() = 0;\n"
            " \n"
