@@ -29,7 +29,6 @@
 #include <QDateTime>
 #include <QDebug>
 #include <QHostAddress>
-#include <QRegExp>
 #include <QTimeZone>
 #include <QJsonArray>
 #include <QJsonDocument>
@@ -47,7 +46,7 @@
 namespace Debugger {
 namespace Internal {
 
-uchar fromhex(uchar c)
+static uchar fromhex(uchar c)
 {
     if (c >= '0' && c <= '9')
         return c - '0';
@@ -58,204 +57,254 @@ uchar fromhex(uchar c)
     return UCHAR_MAX;
 }
 
-void skipCommas(const QChar *&from, const QChar *to)
+// DebuggerOutputParser
+
+DebuggerOutputParser::DebuggerOutputParser(const QString &output)
+    : from(output.begin()), to(output.end())
 {
-    while (*from == ',' && from != to)
+}
+
+void DebuggerOutputParser::skipCommas()
+{
+    while (from != to && *from == ',')
         ++from;
 }
 
-void GdbMi::parseResultOrValue(const QChar *&from, const QChar *to)
+void DebuggerOutputParser::skipSpaces()
 {
     while (from != to && isspace(from->unicode()))
         ++from;
+}
 
-    //qDebug() << "parseResultOrValue: " << QString(from, to - from);
-    parseValue(from, to);
+QString DebuggerOutputParser::readString(const std::function<bool(char)> &isValidChar)
+{
+    QString res;
+    while (from != to && isValidChar(from->unicode()))
+        res += *from++;
+    return res;
+}
+
+int DebuggerOutputParser::readInt()
+{
+    int res = 0;
+    while (from != to && *from >= '0' && *from <= '9') {
+        res *= 10;
+        res += (*from++).unicode() - '0';
+    }
+    return res;
+}
+
+QChar DebuggerOutputParser::readChar()
+{
+    return *from++;
+}
+
+static bool isNameChar(char c)
+{
+    return c != '=' && c != ':' && !isspace(c);
+}
+
+void GdbMi::parseResultOrValue(DebuggerOutputParser &parser)
+{
+    parser.skipSpaces();
+
+    //qDebug() << "parseResultOrValue: " << parser.buffer();
+    parseValue(parser);
+    parser.skipSpaces();
     if (isValid()) {
-        //qDebug() << "no valid result in " << QString(from, to - from);
+        //qDebug() << "no valid result in " << parser.buffer();
         return;
     }
-    if (from == to || *from == '(')
+    if (parser.isAtEnd() || parser.isCurrent('('))
         return;
-    const QChar *ptr = from;
-    while (ptr < to && *ptr != '=' && *ptr != ':') {
-        //qDebug() << "adding" << QChar(*ptr) << "to name";
-        ++ptr;
-    }
-    m_name = QString(from, ptr - from);
-    from = ptr;
-    if (from < to && *from == '=') {
-        ++from;
-        parseValue(from, to);
+
+    m_name = parser.readString(isNameChar);
+
+    if (!parser.isAtEnd() && parser.isCurrent('=')) {
+        parser.advance();
+        parseValue(parser);
     }
 }
 
-QString GdbMi::parseCString(const QChar *&from, const QChar *to)
+// Reads one \ooo entity.
+static bool parseOctalEscapedHelper(DebuggerOutputParser &parser, QByteArray &buffer)
 {
-    QString result;
-    //qDebug() << "parseCString: " << QString(from, to - from);
+    if (parser.remainingChars() < 4)
+        return false;
+    if (!parser.isCurrent('\\'))
+        return false;
+
+    const char c1 = parser.lookAhead(1).unicode();
+    const char c2 = parser.lookAhead(2).unicode();
+    const char c3 = parser.lookAhead(3).unicode();
+    if (!isdigit(c1) || !isdigit(c2) || !isdigit(c3))
+        return false;
+
+    buffer += char((c1 - '0') * 64 + (c2 - '0') * 8 + (c3 - '0'));
+    parser.advance(4);
+    return true;
+}
+
+static bool parseHexEscapedHelper(DebuggerOutputParser &parser, QByteArray &buffer)
+{
+    if (parser.remainingChars() < 4)
+        return false;
+    if (!parser.isCurrent('\\'))
+        return false;
+    if (parser.lookAhead(1) != 'x')
+        return false;
+
+    const char c1 = parser.lookAhead(2).unicode();
+    const char c2 = parser.lookAhead(3).unicode();
+    if (!isxdigit(c1) || !isxdigit(c2))
+        return false;
+
+    buffer += char(16 * fromhex(c1) + fromhex(c2));
+    parser.advance(4);
+    return true;
+}
+
+static void parseSimpleEscape(DebuggerOutputParser &parser, QString &result)
+{
+    if (parser.isAtEnd()) {
+        qDebug() << "MI Parse Error, unterminated backslash escape";
+        return;
+    }
+
+    const QChar c = parser.current();
+    parser.advance();
+    switch (c.unicode()) {
+    case 'a': result += '\a'; break;
+    case 'b': result += '\b'; break;
+    case 'f': result += '\f'; break;
+    case 'n': result += '\n'; break;
+    case 'r': result += '\r'; break;
+    case 't': result += '\t'; break;
+    case 'v': result += '\v'; break;
+    case '"': result += '"'; break;
+    case '\'': result += '\''; break;
+    case '\\': result += '\\'; break;
+    default:
+        qDebug() << "MI Parse Error, unrecognized backslash escape";
+    }
+}
+
+// Reads subsequent \123 or \x12 entities and converts to Utf8,
+// *or* one escaped char, *or* one unescaped char.
+static void parseCharOrEscape(DebuggerOutputParser &parser, QString &result)
+{
+    QByteArray buffer;
+    while (parseOctalEscapedHelper(parser, buffer))
+        ;
+    while (parseHexEscapedHelper(parser, buffer))
+        ;
+
+    if (!buffer.isEmpty()) {
+        result.append(QString::fromUtf8(buffer));
+    } else if (parser.isCurrent('\\')) {
+        parser.advance();
+        parseSimpleEscape(parser, result);
+    } else {
+        result += parser.readChar();
+    }
+}
+
+QString DebuggerOutputParser::readCString()
+{
+    if (isAtEnd())
+        return QString();
+
     if (*from != '"') {
         qDebug() << "MI Parse Error, double quote expected";
         ++from; // So we don't hang
         return QString();
     }
-    const QChar *ptr = from;
-    ++ptr;
-    while (ptr < to) {
-        if (*ptr == '"') {
-            ++ptr;
-            result = QString(from + 1, ptr - from - 2);
-            break;
-        }
-        if (*ptr == '\\') {
-            ++ptr;
-            if (ptr == to) {
-                qDebug() << "MI Parse Error, unterminated backslash escape";
-                from = ptr; // So we don't hang
-                return QString();
-            }
-        }
-        ++ptr;
-    }
-    from = ptr;
 
-    int idx = result.indexOf('\\');
-    if (idx >= 0) {
-        QChar *dst = result.data() + idx;
-        const QChar *src = dst + 1, *end = result.data() + result.length();
-        do {
-            QChar c = *src++;
-            switch (c.unicode()) {
-                case 'a': *dst++ = '\a'; break;
-                case 'b': *dst++ = '\b'; break;
-                case 'f': *dst++ = '\f'; break;
-                case 'n': *dst++ = '\n'; break;
-                case 'r': *dst++ = '\r'; break;
-                case 't': *dst++ = '\t'; break;
-                case 'v': *dst++ = '\v'; break;
-                case '"': *dst++ = '"'; break;
-                case '\\': *dst++ = '\\'; break;
-                case 'x': {
-                        c = *src++;
-                        int chars = 0;
-                        uchar prod = 0;
-                        while (true) {
-                            uchar val = fromhex(c.unicode());
-                            if (val == UCHAR_MAX)
-                                break;
-                            prod = prod * 16 + val;
-                            if (++chars == 3 || src == end)
-                                break;
-                            c = *src++;
-                        }
-                        if (!chars) {
-                            qDebug() << "MI Parse Error, unrecognized hex escape";
-                            return QString();
-                        }
-                        *dst++ = prod;
-                        break;
-                    }
-                default:
-                    {
-                        int chars = 0;
-                        uchar prod = 0;
-                        forever {
-                            if (c < '0' || c > '7') {
-                                --src;
-                                break;
-                            }
-                            prod = prod * 8 + c.unicode() - '0';
-                            if (++chars == 3 || src == end)
-                                break;
-                            c = *src++;
-                        }
-                        if (!chars) {
-                            qDebug() << "MI Parse Error, unrecognized backslash escape";
-                            return QString();
-                        }
-                        *dst++ = prod;
-                    }
-            }
-            while (src != end) {
-                QChar c = *src++;
-                if (c == '\\')
-                    break;
-                *dst++ = c;
-            }
-        } while (src != end);
-        *dst = 0;
-        result.truncate(dst - result.data());
+    ++from; // Skip initial quote.
+    QString result;
+    result.reserve(to - from);
+    while (from < to) {
+        if (*from == '"') {
+            ++from;
+            return result;
+        }
+        parseCharOrEscape(*this, result);
     }
 
-    return result;
+    qDebug() << "MI Parse Error, unfinished string";
+    return QString();
 }
 
-void GdbMi::parseValue(const QChar *&from, const QChar *to)
+void GdbMi::parseValue(DebuggerOutputParser &parser)
 {
-    //qDebug() << "parseValue: " << QString(from, to - from);
-    switch (from->unicode()) {
+    if (parser.isAtEnd())
+        return;
+
+    //qDebug() << "parseValue: " << parser;
+    switch (parser.current().unicode()) {
         case '{':
-            parseTuple(from, to);
+            parseTuple(parser);
             break;
         case '[':
-            parseList(from, to);
+            parseList(parser);
             break;
         case '"':
             m_type = Const;
-            m_data = parseCString(from, to);
+            m_data = parser.readCString();
             break;
         default:
             break;
     }
 }
 
-void GdbMi::parseTuple(const QChar *&from, const QChar *to)
+void GdbMi::parseTuple(DebuggerOutputParser &parser)
 {
-    //qDebug() << "parseTuple: " << QString(from, to - from);
-    //QTC_CHECK(*from == '{');
-    ++from;
-    parseTuple_helper(from, to);
+    //qDebug() << "parseTuple: " << parser.buffer();
+    QTC_CHECK(parser.isCurrent('{'));
+    parser.advance();
+    parseTuple_helper(parser);
 }
 
-void GdbMi::parseTuple_helper(const QChar *&from, const QChar *to)
+void GdbMi::parseTuple_helper(DebuggerOutputParser &parser)
 {
-    skipCommas(from, to);
-    //qDebug() << "parseTuple_helper: " << QString(from, to - from);
+    parser.skipCommas();
+    //qDebug() << "parseTuple_helper: " << parser.buffer();
+    QString buf = parser.buffer();
     m_type = Tuple;
-    while (from < to) {
-        if (*from == '}') {
-            ++from;
+    while (!parser.isAtEnd()) {
+        if (parser.isCurrent('}')) {
+            parser.advance();
             break;
         }
         GdbMi child;
-        child.parseResultOrValue(from, to);
+        child.parseResultOrValue(parser);
         //qDebug() << "\n=======\n" << qPrintable(child.toString()) << "\n========\n";
         if (!child.isValid())
             return;
         m_children.push_back(child);
-        skipCommas(from, to);
+        parser.skipCommas();
     }
 }
 
-void GdbMi::parseList(const QChar *&from, const QChar *to)
+void GdbMi::parseList(DebuggerOutputParser &parser)
 {
-    //qDebug() << "parseList: " << QString(from, to - from);
-    //QTC_CHECK(*from == '[');
-    ++from;
+    //qDebug() << "parseList: " << parser.buffer();
+    QTC_CHECK(parser.isCurrent('['));
+    parser.advance();
     m_type = List;
-    skipCommas(from, to);
-    while (from < to) {
-        if (*from == ']') {
-            ++from;
+    parser.skipCommas();
+    while (!parser.isAtEnd()) {
+        if (parser.isCurrent(']')) {
+            parser.advance();
             break;
         }
         GdbMi child;
-        child.parseResultOrValue(from, to);
+        child.parseResultOrValue(parser);
         if (child.isValid()) {
             m_children.push_back(child);
-            skipCommas(from, to);
+            parser.skipCommas();
         } else {
-            ++from;
+            parser.advance();
         }
     }
 }
@@ -302,9 +351,9 @@ QString GdbMi::escapeCString(const QString &ba)
             default:
                 if (c < 32 || c == 127) {
                     ret += '\\';
-                    ret += ('0' + (c >> 6));
-                    ret += ('0' + ((c >> 3) & 7));
-                    ret += ('0' + (c & 7));
+                    ret += QLatin1Char('0' + (c >> 6));
+                    ret += QLatin1Char('0' + ((c >> 3) & 7));
+                    ret += QLatin1Char('0' + (c & 7));
                 } else {
                     ret += c;
                 }
@@ -360,16 +409,14 @@ QString GdbMi::toString(bool multiline, int indent) const
 
 void GdbMi::fromString(const QString &ba)
 {
-    const QChar *from = ba.constBegin();
-    const QChar *to = ba.constEnd();
-    parseResultOrValue(from, to);
+    DebuggerOutputParser parser(ba);
+    parseResultOrValue(parser);
 }
 
 void GdbMi::fromStringMultiple(const QString &ba)
 {
-    const QChar *from = ba.constBegin();
-    const QChar *to = ba.constEnd();
-    parseTuple_helper(from, to);
+    DebuggerOutputParser parser(ba);
+    parseTuple_helper(parser);
 }
 
 const GdbMi &GdbMi::operator[](const char *name) const
@@ -457,7 +504,13 @@ void extractGdbVersion(const QString &msg,
     QString build;
     bool inClean = true;
     bool inParenthesis = false;
-    for (QChar c : msg) {
+
+    int gdbMsgBegin = msg.indexOf("GNU gdb");
+    if (gdbMsgBegin == -1)
+      gdbMsgBegin = 0;
+
+    for (int i = gdbMsgBegin, gdbMsgSize = msg.size(); i < gdbMsgSize; ++i) {
+        QChar c = msg.at(i);
         if (inClean && !cleaned.isEmpty() && c != dot && (c.isPunct() || c.isSpace()))
             inClean = false;
         if (ignoreParenthesisContent) {
@@ -509,7 +562,7 @@ static QString quoteUnprintableLatin1(const QString &ba)
     for (int i = 0, n = ba.size(); i != n; ++i) {
         const unsigned char c = ba.at(i).unicode();
         if (isprint(c)) {
-            res += c;
+            res += ba.at(i);
         } else {
             qsnprintf(buf, sizeof(buf) - 1, "\\%x", int(c));
             res += QLatin1String(buf);
@@ -652,7 +705,7 @@ QString decodeData(const QString &ba, const QString &encoding)
         case DebuggerEncoding::JulianDateAndMillisecondsSinceMidnight: {
             const int p = ba.indexOf('/');
             const QDate date = dateFromData(ba.left(p).toInt());
-            const QTime time = timeFromData(ba.midRef(p + 1 ).toInt());
+            const QTime time = timeFromData(ba.mid(p + 1).toInt());
             const QDateTime dateTime = QDateTime(date, time);
             return dateTime.isValid() ? dateTime.toString(Qt::TextDate) : "(invalid)";
         }
@@ -661,7 +714,9 @@ QString decodeData(const QString &ba, const QString &encoding)
             qDebug("not implemented"); // Only used in Arrays, see watchdata.cpp
             return QString();
         case DebuggerEncoding::HexEncodedFloat: {
-            const QByteArray s = QByteArray::fromHex(ba.toUtf8());
+            QByteArray s = QByteArray::fromHex(ba.toUtf8());
+            if (s.size() < enc.size)
+                s.prepend(QByteArray(enc.size - s.size(), '\0'));
             if (enc.size == 4) {
                 union { char c[4]; float f; } u = {{s[3], s[2], s[1], s[0]}};
                 return QString::number(u.f);
@@ -693,15 +748,15 @@ QString decodeData(const QString &ba, const QString &encoding)
 
             qint64 msecs = ba.left(p0).toLongLong();
             ++p0;
-            Qt::TimeSpec spec = Qt::TimeSpec(ba.midRef(p0, p1 - p0).toInt());
+            Qt::TimeSpec spec = Qt::TimeSpec(ba.mid(p0, p1 - p0).toInt());
             ++p1;
-            qulonglong offset = ba.midRef(p1, p2 - p1).toInt();
+            qulonglong offset = ba.mid(p1, p2 - p1).toInt();
             ++p2;
             QByteArray timeZoneId = QByteArray::fromHex(ba.mid(p2, p3 - p2).toUtf8());
             ++p3;
-            int status = ba.midRef(p3, p4 - p3).toInt();
+            int status = ba.mid(p3, p4 - p3).toInt();
             ++p4;
-            int tiVersion = ba.midRef(p4).toInt();
+            int tiVersion = ba.mid(p4).toInt();
 
             QDate date;
             QTime time;
@@ -805,6 +860,11 @@ void DebuggerCommand::arg(const char *name, const QJsonValue &value)
     args = addToJsonObject(args, name, value);
 }
 
+void DebuggerCommand::arg(const char *name, const Utils::FilePath &filePath)
+{
+    args = addToJsonObject(args, name, filePath.toString());
+}
+
 static QJsonValue translateJsonToPython(const QJsonValue &value)
 {
     // TODO: Verify that this covers all incompatibilities between python and json,
@@ -851,9 +911,9 @@ QString DebuggerCommand::argsToString() const
 
 DebuggerEncoding::DebuggerEncoding(const QString &data)
 {
-    const QVector<QStringRef> l = data.splitRef(':');
+    const QStringList l = data.split(':');
 
-    const QStringRef &t = l.at(0);
+    const QString &t = l.at(0);
     if (t == "latin1") {
         type = HexEncodedLatin1;
         size = 1;
