@@ -37,7 +37,6 @@
 #include <QHelpEngine>
 #include <QStyle>
 #include <QTimer>
-#include <QtMac>
 #include <QToolTip>
 #include <QUrl>
 #include <QVBoxLayout>
@@ -336,7 +335,6 @@ static NSMenuItem *menuItem(NSURL *url, id target, SEL action, const QString &ti
         case WebMenuItemTagGoBack:
         case WebMenuItemTagGoForward:
         case WebMenuItemTagStop:
-        case WebMenuItemTagReload:
         case WebMenuItemTagOther:
         case WebMenuItemTagSearchInSpotlight:
         case WebMenuItemTagSearchWeb:
@@ -536,16 +534,6 @@ MacWebKitHelpViewer::~MacWebKitHelpViewer()
 
 }
 
-QFont MacWebKitHelpViewer::viewerFont() const
-{
-    @autoreleasepool {
-        WebPreferences *preferences = m_widget->webView().preferences;
-        QString family = QString::fromNSString([preferences standardFontFamily]);
-        int size = [preferences defaultFontSize];
-        return QFont(family, size);
-    }
-}
-
 void MacWebKitHelpViewer::setViewerFont(const QFont &font)
 {
     @autoreleasepool {
@@ -555,34 +543,6 @@ void MacWebKitHelpViewer::setViewerFont(const QFont &font)
     }
 }
 
-void MacWebKitHelpViewer::scaleUp()
-{
-    @autoreleasepool {
-        m_widget->webView().textSizeMultiplier += 0.1f;
-    }
-}
-
-void MacWebKitHelpViewer::scaleDown()
-{
-    @autoreleasepool {
-        m_widget->webView().textSizeMultiplier = qMax(0.1f, m_widget->webView().textSizeMultiplier - 0.1f);
-    }
-}
-
-void MacWebKitHelpViewer::resetScale()
-{
-    @autoreleasepool {
-        m_widget->webView().textSizeMultiplier = 1.0f;
-    }
-}
-
-qreal MacWebKitHelpViewer::scale() const
-{
-    @autoreleasepool {
-        return m_widget->webView().textSizeMultiplier;
-    }
-}
-
 void MacWebKitHelpViewer::setScale(qreal scale)
 {
     m_widget->webView().textSizeMultiplier = (scale <= 0.0 ? 1.0 : scale);
