@@ -27,6 +27,9 @@
 #include "mcusupportoptions.h"
 #include "mcusupportsdk.h"
 
+#include <baremetal/baremetalconstants.h>
+#include <projectexplorer/toolchain.h>
+#include <projectexplorer/toolchainmanager.h>
 #include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
 #include <utils/fileutils.h>
@@ -78,6 +81,11 @@ static McuToolChainPackage *createGccToolChainPackage()
     return new McuToolChainPackage({}, {}, {}, {}, McuToolChainPackage::TypeGCC);
 }
 
+static McuToolChainPackage *createUnsupportedToolChainPackage()
+{
+    return new McuToolChainPackage({}, {}, {}, {}, McuToolChainPackage::TypeUnsupported);
+}
+
 static McuToolChainPackage *createArmGccPackage()
 {
     const char envVar[] = "ARMGCC_DIR";
@@ -126,6 +134,36 @@ static McuToolChainPackage *createGhsToolchainPackage()
     return result;
 }
 
+static McuToolChainPackage *createIarToolChainPackage()
+{
+    const char envVar[] = "IAR_ARM_COMPILER_DIR";
+
+    QString defaultPath;
+    if (qEnvironmentVariableIsSet(envVar))
+        defaultPath = qEnvironmentVariable(envVar);
+    else {
+        const ProjectExplorer::ToolChain *tc =
+                ProjectExplorer::ToolChainManager::toolChain([](const ProjectExplorer::ToolChain *t) {
+            return  t->typeId() == BareMetal::Constants::IAREW_TOOLCHAIN_TYPEID;
+        });
+        if (tc) {
+            const Utils::FilePath compilerExecPath = tc->compilerCommand();
+            defaultPath = compilerExecPath.parentDir().parentDir().toString();
+        }
+        else
+            defaultPath = QDir::homePath();
+    }
+
+    auto result = new McuToolChainPackage(
+                "IAR ARM Compiler",
+                defaultPath,
+                Utils::HostOsInfo::withExecutableSuffix("bin/iccarm"),
+                "IARToolchain",
+                McuToolChainPackage::TypeIAR);
+    result->setEnvironmentVariableName(envVar);
+    return result;
+}
+
 static McuPackage *createRGLPackage()
 {
     const char envVar[] = "RGL_DIR";
@@ -310,8 +348,10 @@ struct McuTargetFactory
     QVector<McuPackage *> getMcuPackages() const
     {
         QVector<McuPackage *> packages;
-        packages.append(boardSdkPkgs.values().toVector());
-        packages.append(freeRTOSPkgs.values().toVector());
+        for (auto *package : qAsConst(boardSdkPkgs))
+            packages.append(package);
+        for (auto *package : qAsConst(freeRTOSPkgs))
+            packages.append(package);
         return packages;
     }
 
@@ -321,6 +361,8 @@ protected:
     {
         QVector<McuTarget *> mcuTargets;
         McuToolChainPackage *tcPkg = tcPkgs.value(desc.toolchainId);
+        if (!tcPkg)
+            tcPkg = createUnsupportedToolChainPackage();
         for (auto os : {McuTarget::OS::BareMetal, McuTarget::OS::FreeRTOS}) {
             for (int colorDepth : desc.colorDepths) {
                 QVector<McuPackage*> required3rdPartyPkgs = { tcPkg };
@@ -366,6 +408,8 @@ protected:
     QVector<McuTarget *> createDesktopTargetsLegacy(const McuTargetDescription& desc)
     {
         McuToolChainPackage *tcPkg = tcPkgs.value(desc.toolchainId);
+        if (!tcPkg)
+            tcPkg = createUnsupportedToolChainPackage();
         const auto platform = McuTarget::Platform{ desc.platform, desc.platformName, desc.platformVendor };
         auto desktopTarget = new McuTarget(QVersionNumber::fromString(desc.qulVersion),
                                            platform, McuTarget::OS::Desktop, {}, tcPkg);
@@ -385,10 +429,14 @@ protected:
 
         QVector<McuTarget *> mcuTargets;
         McuToolChainPackage *tcPkg = tcPkgs.value(desc.toolchainId);
+        if (!tcPkg)
+            tcPkg = createUnsupportedToolChainPackage();
         for (int colorDepth : desc.colorDepths) {
             QVector<McuPackage*> required3rdPartyPkgs;
             // Desktop toolchains don't need any additional settings
-            if (tcPkg && !tcPkg->isDesktopToolchain())
+            if (tcPkg
+                && !tcPkg->isDesktopToolchain()
+                && tcPkg->type() != McuToolChainPackage::TypeUnsupported)
                 required3rdPartyPkgs.append(tcPkg);
 
             // Add setting specific to platform IDE
@@ -440,6 +488,7 @@ static QVector<McuTarget *> targetsFromDescriptions(const QList<McuTargetDescrip
     const QHash<QString, McuToolChainPackage *> tcPkgs = {
         {{"armgcc"}, createArmGccPackage()},
         {{"greenhills"}, createGhsToolchainPackage()},
+        {{"iar"}, createIarToolChainPackage()},
         {{"msvc"}, createMsvcToolChainPackage()},
         {{"gcc"}, createGccToolChainPackage()},
     };
@@ -459,15 +508,21 @@ static QVector<McuTarget *> targetsFromDescriptions(const QList<McuTargetDescrip
 
     packages->append(Utils::transform<QVector<McuPackage *> >(
                          tcPkgs.values(), [&](McuToolChainPackage *tcPkg) { return tcPkg; }));
-    packages->append(vendorPkgs.values().toVector());
+    for (auto *package : vendorPkgs)
+        packages->append(package);
     packages->append(targetFactory.getMcuPackages());
 
     return  mcuTargets;
 }
 
+Utils::FilePath kitsPath(const Utils::FilePath &dir)
+{
+    return dir + "/kits/";
+}
+
 static QFileInfoList targetDescriptionFiles(const Utils::FilePath &dir)
 {
-    const QDir kitsDir(dir.toString() + "/kits/", "*.json");
+    const QDir kitsDir(kitsPath(dir).toString(), "*.json");
     return kitsDir.entryInfoList();
 }
 
@@ -504,16 +559,31 @@ void targetsAndPackages(const Utils::FilePath &dir, QVector<McuPackage *> *packa
 {
     QList<McuTargetDescription> descriptions;
 
-    for (const QFileInfo &fileInfo : targetDescriptionFiles(dir)) {
+    auto descriptionFiles = targetDescriptionFiles(dir);
+    for (const QFileInfo &fileInfo : descriptionFiles) {
         QFile file(fileInfo.absoluteFilePath());
         if (!file.open(QFile::ReadOnly))
             continue;
         const McuTargetDescription desc = parseDescriptionJson(file.readAll());
-        if (QVersionNumber::fromString(desc.qulVersion) < McuSupportOptions::minimalQulVersion())
-            return; // Invalid version means invalid SDK installation.
+        if (QVersionNumber::fromString(desc.qulVersion) < McuSupportOptions::minimalQulVersion()) {
+            auto pth = Utils::FilePath::fromString(fileInfo.filePath());
+            printMessage(McuTarget::tr("Skipped %1 - Unsupported version \"%2\" (should be >= %3)")
+                         .arg(
+                             QDir::toNativeSeparators(pth.fileNameWithPathComponents(1)),
+                             desc.qulVersion,
+                             McuSupportOptions::minimalQulVersion().toString()),
+                             false);
+            continue;
+        }
         descriptions.append(desc);
     }
 
+    // No valid description means invalid SDK installation.
+    if (descriptions.empty() && kitsPath(dir).exists()) {
+        printMessage(McuTarget::tr("No valid kit descriptions found at %1.").arg(kitsPath(dir).toUserOutput()), true);
+        return;
+    }
+
     // Workaround for missing JSON file for Desktop target.
     // Desktop JSON file is shipped starting from Qul 1.5.
     // This whole section could be removed when minimalQulVersion will reach 1.5 or above
@@ -523,13 +593,18 @@ void targetsAndPackages(const Utils::FilePath &dir, QVector<McuPackage *> *packa
         });
 
         if (!hasDesktopDescription) {
-            Utils::FilePath desktopLib;
-            if (Utils::HostOsInfo::isWindowsHost())
-                desktopLib = dir / "lib/QulQuickUltralite_QT_32bpp_Windows_Release.lib";
-            else
-                desktopLib = dir / "lib/libQulQuickUltralite_QT_32bpp_Linux_Debug.a";
+            QVector<Utils::FilePath> desktopLibs;
+            if (Utils::HostOsInfo::isWindowsHost()) {
+                desktopLibs << dir / "lib/QulQuickUltralite_QT_32bpp_Windows_Release.lib"; // older versions of QUL (<1.5?)
+                desktopLibs << dir / "lib/QulQuickUltralitePlatform_QT_32bpp_Windows_msvc_Release.lib"; // newer versions of QUL
+            } else {
+                desktopLibs << dir / "lib/libQulQuickUltralite_QT_32bpp_Linux_Debug.a"; // older versions of QUL (<1.5?)
+                desktopLibs << dir / "lib/libQulQuickUltralitePlatform_QT_32bpp_Linux_gnu_Debug.a"; // newer versions of QUL
+            }
 
-            if (desktopLib.exists()) {
+            if (Utils::anyOf(desktopLibs, [](const Utils::FilePath &desktopLib) {
+                             return desktopLib.exists(); })
+                    ) {
                 McuTargetDescription desktopDescription;
                 desktopDescription.qulVersion = descriptions.empty() ?
                             McuSupportOptions::minimalQulVersion().toString()
@@ -541,6 +616,13 @@ void targetsAndPackages(const Utils::FilePath &dir, QVector<McuPackage *> *packa
                 desktopDescription.toolchainId = Utils::HostOsInfo::isWindowsHost() ? QString("msvc") : QString("gcc");
                 desktopDescription.type = McuTargetDescription::TargetType::Desktop;
                 descriptions.prepend(desktopDescription);
+            } else {
+                if (dir.exists())
+                    printMessage(McuTarget::tr("Skipped creating fallback desktop kit: Could not find any of %1.")
+                                 .arg(Utils::transform(desktopLibs, [](const auto &path) {
+                                    return QDir::toNativeSeparators(path.fileNameWithPathComponents(1));
+                                 }).toList().join(" or ")),
+                                 false);
             }
         }
     }
