@@ -37,7 +37,7 @@
 #include "qmt/project_controller/projectcontroller.h"
 #include "qmt/project/project.h"
 
-#include <coreplugin/id.h>
+#include <utils/id.h>
 #include <utils/fileutils.h>
 
 #include <QFileInfo>
@@ -69,7 +69,7 @@ ModelDocument::~ModelDocument()
 Core::IDocument::OpenResult ModelDocument::open(QString *errorString, const QString &fileName,
                                                 const QString &realFileName)
 {
-    Q_UNUSED(fileName);
+    Q_UNUSED(fileName)
 
     OpenResult result = load(errorString, realFileName);
     return result;
@@ -96,7 +96,7 @@ bool ModelDocument::save(QString *errorString, const QString &name, bool autoSav
     if (autoSave) {
         d->documentController->projectController()->setModified();
     } else {
-        setFilePath(Utils::FileName::fromString(d->documentController->projectController()->project()->fileName()));
+        setFilePath(Utils::FilePath::fromString(d->documentController->projectController()->project()->fileName()));
         emit changed();
     }
 
@@ -121,12 +121,9 @@ bool ModelDocument::isSaveAsAllowed() const
 bool ModelDocument::reload(QString *errorString, Core::IDocument::ReloadFlag flag,
                            Core::IDocument::ChangeType type)
 {
+    Q_UNUSED(type)
     if (flag == FlagIgnore)
         return true;
-    if (type == TypePermissions) {
-        emit changed();
-        return true;
-    }
     try {
         d->documentController->loadProject(filePath().toString());
     } catch (const qmt::FileNotFoundException &ex) {
@@ -152,7 +149,7 @@ Core::IDocument::OpenResult ModelDocument::load(QString *errorString, const QStr
 
     try {
         d->documentController->loadProject(fileName);
-        setFilePath(Utils::FileName::fromString(d->documentController->projectController()->project()->fileName()));
+        setFilePath(Utils::FilePath::fromString(d->documentController->projectController()->project()->fileName()));
     } catch (const qmt::FileNotFoundException &ex) {
         *errorString = ex.errorMessage();
         return OpenResult::ReadError;
