@@ -27,18 +27,13 @@
 
 #include "abstractprocessstep.h"
 
+#include <utils/aspects.h>
 #include <utils/fileutils.h>
 
-QT_FORWARD_DECLARE_CLASS(QListWidgetItem);
-
 namespace Utils { class Environment; }
 
 namespace ProjectExplorer {
 
-namespace Internal {
-namespace Ui { class MakeStep; }
-} // namespace Internal
-
 class PROJECTEXPLORER_EXPORT MakeStep : public ProjectExplorer::AbstractProcessStep
 {
     Q_OBJECT
@@ -48,15 +43,15 @@ public:
         Display,
         Execution
     };
-    explicit MakeStep(ProjectExplorer::BuildStepList *parent, Core::Id id);
+    explicit MakeStep(ProjectExplorer::BuildStepList *parent, Utils::Id id);
 
-    void setBuildTarget(const QString &buildTarget);
     void setAvailableBuildTargets(const QStringList &buildTargets);
+    void setSelectedBuildTarget(const QString &buildTarget);
 
     bool init() override;
-    ProjectExplorer::BuildStepConfigWidget *createConfigWidget() override;
-    bool buildsTarget(const QString &target) const;
-    void setBuildTarget(const QString &target, bool on);
+    void setupOutputFormatter(Utils::OutputFormatter *formatter) override;
+    QWidget *createConfigWidget() override;
+
     QStringList availableTargets() const;
     QString userArguments() const;
     void setUserArguments(const QString &args);
@@ -65,9 +60,6 @@ public:
     Utils::FilePath makeExecutable() const;
     Utils::CommandLine effectiveMakeCommand(MakeCommandType type) const;
 
-    void setClean(bool clean);
-    bool isClean() const;
-
     static QString defaultDisplayName();
 
     Utils::FilePath defaultMakeCommand() const;
@@ -76,58 +68,39 @@ public:
 
     virtual bool isJobCountSupported() const;
     int jobCount() const;
-    void setJobCount(int count);
     bool jobCountOverridesMakeflags() const;
-    void setJobCountOverrideMakeflags(bool override);
     bool makeflagsContainsJobCount() const;
     bool userArgsContainsJobCount() const;
     bool makeflagsJobCountMismatch() const;
 
     bool disablingForSubdirsSupported() const { return m_disablingForSubDirsSupported; }
-    bool enabledForSubDirs() const { return m_enabledForSubDirs; }
-    void setEnabledForSubDirs(bool enabled) { m_enabledForSubDirs = enabled; }
+    bool enabledForSubDirs() const;
 
-    Utils::Environment environment(BuildConfiguration *bc) const;
+    Utils::Environment makeEnvironment() const;
+
+    // FIXME: All unused, remove in 4.15.
+    void setBuildTarget(const QString &buildTarget) { setSelectedBuildTarget(buildTarget); }
+    bool buildsTarget(const QString &target) const;
+    void setBuildTarget(const QString &target, bool on);
 
 protected:
-    bool fromMap(const QVariantMap &map) override;
     void supportDisablingForSubdirs() { m_disablingForSubDirsSupported = true; }
     virtual QStringList displayArguments() const;
 
 private:
-    QVariantMap toMap() const override;
     static int defaultJobCount();
     QStringList jobArguments() const;
 
-    QStringList m_buildTargets;
-    QStringList m_availableTargets;
-    QString m_userArguments;
-    Utils::FilePath m_makeCommand;
-    int m_userJobCount = 4;
-    bool m_overrideMakeflags = false;
-    bool m_clean = false;
+    Utils::MultiSelectionAspect *m_buildTargetsAspect = nullptr;
+    QStringList m_availableTargets; // FIXME: Unused, remove in 4.15.
+    Utils::StringAspect *m_makeCommandAspect = nullptr;
+    Utils::StringAspect *m_userArgumentsAspect = nullptr;
+    Utils::AspectContainer *m_jobCountContainer = nullptr;
+    Utils::IntegerAspect *m_userJobCountAspect = nullptr;
+    Utils::BoolAspect *m_overrideMakeflagsAspect = nullptr;
+    Utils::BoolAspect *m_disabledForSubdirsAspect = nullptr;
+    Utils::TextDisplay *m_nonOverrideWarning = nullptr;
     bool m_disablingForSubDirsSupported = false;
-    bool m_enabledForSubDirs = true;
 };
 
-class PROJECTEXPLORER_EXPORT MakeStepConfigWidget : public ProjectExplorer::BuildStepConfigWidget
-{
-    Q_OBJECT
-
-public:
-    explicit MakeStepConfigWidget(MakeStep *makeStep);
-    ~MakeStepConfigWidget() override;
-
-private:
-    void itemChanged(QListWidgetItem *item);
-    void makeLineEditTextEdited();
-    void makeArgumentsLineEditTextEdited();
-    void updateDetails();
-    void setUserJobCountVisible(bool visible);
-    void setUserJobCountEnabled(bool enabled);
-
-    Internal::Ui::MakeStep *m_ui;
-    MakeStep *m_makeStep;
-};
-
-} // namespace GenericProjectManager
+} // namespace ProjectExplorer
