@@ -34,6 +34,7 @@
 
 #include <utils/detailsbutton.h>
 #include <utils/fancylineedit.h>
+#include <utils/layoutbuilder.h>
 #include <utils/pathchooser.h>
 #include <utils/qtcprocess.h>
 #include <utils/utilsicons.h>
@@ -51,6 +52,12 @@ namespace ProjectExplorer {
 
 /*!
     \class ProjectExplorer::TerminalAspect
+    \inmodule QtCreator
+
+    \brief The TerminalAspect class lets a user specify that an executable
+    should be run in a separate terminal.
+
+    The initial value is provided as a hint from the build systems.
 */
 
 TerminalAspect::TerminalAspect()
@@ -63,12 +70,15 @@ TerminalAspect::TerminalAspect()
             this, &TerminalAspect::calculateUseTerminal);
 }
 
+/*!
+    \reimp
+*/
 void TerminalAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!m_checkBox);
     m_checkBox = new QCheckBox(tr("Run in terminal"));
     m_checkBox->setChecked(m_useTerminal);
-    builder.addItems(QString(), m_checkBox.data());
+    builder.addItems({{}, m_checkBox.data()});
     connect(m_checkBox.data(), &QAbstractButton::clicked, this, [this] {
         m_userSet = true;
         m_useTerminal = m_checkBox->isChecked();
@@ -76,6 +86,9 @@ void TerminalAspect::addToLayout(LayoutBuilder &builder)
     });
 }
 
+/*!
+    \reimp
+*/
 void TerminalAspect::fromMap(const QVariantMap &map)
 {
     if (map.contains(settingsKey())) {
@@ -89,6 +102,9 @@ void TerminalAspect::fromMap(const QVariantMap &map)
         m_checkBox->setChecked(m_useTerminal);
 }
 
+/*!
+    \reimp
+*/
 void TerminalAspect::toMap(QVariantMap &data) const
 {
     if (m_userSet)
@@ -113,17 +129,26 @@ void TerminalAspect::calculateUseTerminal()
         m_checkBox->setChecked(m_useTerminal);
 }
 
+/*!
+    Returns whether a separate terminal should be used.
+*/
 bool TerminalAspect::useTerminal() const
 {
     return m_useTerminal;
 }
 
+/*!
+    Sets the initial value to \a hint.
+*/
 void TerminalAspect::setUseTerminalHint(bool hint)
 {
     m_useTerminalHint = hint;
     calculateUseTerminal();
 }
 
+/*!
+    Returns whether the user set the value.
+*/
 bool TerminalAspect::isUserSet() const
 {
     return m_userSet;
@@ -131,6 +156,10 @@ bool TerminalAspect::isUserSet() const
 
 /*!
     \class ProjectExplorer::WorkingDirectoryAspect
+    \inmodule QtCreator
+
+    \brief The WorkingDirectoryAspect class lets the user specify a
+    working directory for running the executable.
 */
 
 WorkingDirectoryAspect::WorkingDirectoryAspect()
@@ -140,6 +169,9 @@ WorkingDirectoryAspect::WorkingDirectoryAspect()
     setSettingsKey("RunConfiguration.WorkingDirectory");
 }
 
+/*!
+    \reimp
+*/
 void WorkingDirectoryAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!m_chooser);
@@ -148,10 +180,10 @@ void WorkingDirectoryAspect::addToLayout(LayoutBuilder &builder)
     m_chooser->setExpectedKind(Utils::PathChooser::Directory);
     m_chooser->setPromptDialogTitle(tr("Select Working Directory"));
     m_chooser->setBaseDirectory(m_defaultWorkingDirectory);
-    m_chooser->setFileName(m_workingDirectory.isEmpty() ? m_defaultWorkingDirectory : m_workingDirectory);
+    m_chooser->setFilePath(m_workingDirectory.isEmpty() ? m_defaultWorkingDirectory : m_workingDirectory);
     connect(m_chooser.data(), &PathChooser::pathChanged, this,
             [this]() {
-                m_workingDirectory = m_chooser->rawFileName();
+                m_workingDirectory = m_chooser->rawFilePath();
                 m_resetButton->setEnabled(m_workingDirectory != m_defaultWorkingDirectory);
             });
 
@@ -168,44 +200,50 @@ void WorkingDirectoryAspect::addToLayout(LayoutBuilder &builder)
         m_chooser->setEnvironment(m_envAspect->environment());
     }
 
-    builder.addItems(tr("Working directory:"), m_chooser.data(), m_resetButton.data());
+    builder.addItems({tr("Working directory:"), m_chooser.data(), m_resetButton.data()});
 }
 
-void WorkingDirectoryAspect::acquaintSiblings(const ProjectConfigurationAspects &siblings)
+void WorkingDirectoryAspect::acquaintSiblings(const BaseAspects &siblings)
 {
     m_envAspect = siblings.aspect<EnvironmentAspect>();
 }
 
-QString WorkingDirectoryAspect::keyForDefaultWd() const
-{
-    return settingsKey() + ".default";
-}
-
 void WorkingDirectoryAspect::resetPath()
 {
-    m_chooser->setFileName(m_defaultWorkingDirectory);
+    m_chooser->setFilePath(m_defaultWorkingDirectory);
 }
 
+/*!
+    \reimp
+*/
 void WorkingDirectoryAspect::fromMap(const QVariantMap &map)
 {
     m_workingDirectory = FilePath::fromString(map.value(settingsKey()).toString());
-    m_defaultWorkingDirectory = FilePath::fromString(map.value(keyForDefaultWd()).toString());
+    m_defaultWorkingDirectory = FilePath::fromString(map.value(settingsKey() + ".default").toString());
 
     if (m_workingDirectory.isEmpty())
         m_workingDirectory = m_defaultWorkingDirectory;
 
     if (m_chooser)
-        m_chooser->setFileName(m_workingDirectory.isEmpty() ? m_defaultWorkingDirectory : m_workingDirectory);
+        m_chooser->setFilePath(m_workingDirectory.isEmpty() ? m_defaultWorkingDirectory : m_workingDirectory);
 }
 
+/*!
+    \reimp
+*/
 void WorkingDirectoryAspect::toMap(QVariantMap &data) const
 {
     const QString wd = m_workingDirectory == m_defaultWorkingDirectory
         ? QString() : m_workingDirectory.toString();
-    data.insert(settingsKey(), wd);
-    data.insert(keyForDefaultWd(), m_defaultWorkingDirectory.toString());
+    saveToMap(data, wd, QString());
+    saveToMap(data, m_defaultWorkingDirectory.toString(), QString(), ".default");
 }
 
+/*!
+    Returns the selected directory.
+
+    Macros in the value are expanded using \a expander.
+*/
 FilePath WorkingDirectoryAspect::workingDirectory(const MacroExpander *expander) const
 {
     const Utils::Environment env = m_envAspect ? m_envAspect->environment()
@@ -221,11 +259,19 @@ FilePath WorkingDirectoryAspect::defaultWorkingDirectory() const
     return m_defaultWorkingDirectory;
 }
 
+/*!
+    Returns the selected directory.
+
+    Macros in the value are not expanded.
+*/
 FilePath WorkingDirectoryAspect::unexpandedWorkingDirectory() const
 {
     return m_workingDirectory;
 }
 
+/*!
+    Sets the default value to \a defaultWorkingDir.
+*/
 void WorkingDirectoryAspect::setDefaultWorkingDirectory(const FilePath &defaultWorkingDir)
 {
     if (defaultWorkingDir == m_defaultWorkingDirectory)
@@ -238,11 +284,14 @@ void WorkingDirectoryAspect::setDefaultWorkingDirectory(const FilePath &defaultW
 
     if (m_workingDirectory.isEmpty() || m_workingDirectory == oldDefaultDir) {
         if (m_chooser)
-            m_chooser->setFileName(m_defaultWorkingDirectory);
+            m_chooser->setFilePath(m_defaultWorkingDirectory);
         m_workingDirectory = defaultWorkingDir;
     }
 }
 
+/*!
+    \internal
+*/
 PathChooser *WorkingDirectoryAspect::pathChooser() const
 {
     return m_chooser;
@@ -251,6 +300,10 @@ PathChooser *WorkingDirectoryAspect::pathChooser() const
 
 /*!
     \class ProjectExplorer::ArgumentsAspect
+    \inmodule QtCreator
+
+    \brief The ArgumentsAspect class lets a user specify command line
+    arguments for an executable.
 */
 
 ArgumentsAspect::ArgumentsAspect()
@@ -258,8 +311,14 @@ ArgumentsAspect::ArgumentsAspect()
     setDisplayName(tr("Arguments"));
     setId("ArgumentsAspect");
     setSettingsKey("RunConfiguration.Arguments");
+    m_labelText = tr("Command line arguments:");
 }
 
+/*!
+    Returns the main value of this aspect.
+
+    Macros in the value are expanded using \a expander.
+*/
 QString ArgumentsAspect::arguments(const MacroExpander *expander) const
 {
     QTC_ASSERT(expander, return m_arguments);
@@ -272,16 +331,24 @@ QString ArgumentsAspect::arguments(const MacroExpander *expander) const
     return expanded;
 }
 
+/*!
+    Returns the main value of this aspect.
+
+    Macros in the value are not expanded.
+*/
 QString ArgumentsAspect::unexpandedArguments() const
 {
     return m_arguments;
 }
 
+/*!
+    Sets the main value of this aspect to \a arguments.
+*/
 void ArgumentsAspect::setArguments(const QString &arguments)
 {
     if (arguments != m_arguments) {
         m_arguments = arguments;
-        emit argumentsChanged(arguments);
+        emit changed();
     }
     if (m_chooser && m_chooser->text() != arguments)
         m_chooser->setText(arguments);
@@ -289,6 +356,37 @@ void ArgumentsAspect::setArguments(const QString &arguments)
         m_multiLineChooser->setPlainText(arguments);
 }
 
+/*!
+    Sets the displayed label text to \a labelText.
+*/
+void ArgumentsAspect::setLabelText(const QString &labelText)
+{
+    m_labelText = labelText;
+}
+
+/*!
+    Adds a button to reset the main value of this aspect to the value
+    computed by \a resetter.
+*/
+void ArgumentsAspect::setResetter(const std::function<QString()> &resetter)
+{
+    m_resetter = resetter;
+}
+
+/*!
+    Resets the main value of this aspect.
+*/
+void ArgumentsAspect::resetArguments()
+{
+    QString arguments;
+    if (m_resetter)
+        arguments = m_resetter();
+    setArguments(arguments);
+}
+
+/*!
+    \reimp
+*/
 void ArgumentsAspect::fromMap(const QVariantMap &map)
 {
     QVariant args = map.value(settingsKey());
@@ -308,12 +406,18 @@ void ArgumentsAspect::fromMap(const QVariantMap &map)
         m_multiLineChooser->setPlainText(m_arguments);
 }
 
+/*!
+    \reimp
+*/
 void ArgumentsAspect::toMap(QVariantMap &map) const
 {
-    map.insert(settingsKey(), m_arguments);
-    map.insert(settingsKey() + ".multi", m_multiLine);
+    saveToMap(map, m_arguments, QString());
+    saveToMap(map, m_multiLine, false, ".multi");
 }
 
+/*!
+    \internal
+*/
 QWidget *ArgumentsAspect::setupChooser()
 {
     if (m_multiLine) {
@@ -334,10 +438,12 @@ QWidget *ArgumentsAspect::setupChooser()
     return m_chooser.data();
 }
 
+/*!
+    \reimp
+*/
 void ArgumentsAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!m_chooser && !m_multiLineChooser && !m_multiLineButton);
-    builder.addItem(tr("Command line arguments:"));
 
     const auto container = new QWidget;
     const auto containerLayout = new QHBoxLayout(container);
@@ -370,11 +476,28 @@ void ArgumentsAspect::addToLayout(LayoutBuilder &builder)
     containerLayout->addWidget(m_multiLineButton);
     containerLayout->setAlignment(m_multiLineButton, Qt::AlignTop);
 
-    builder.addItem(container);
+    if (m_resetter) {
+        m_resetButton = new QToolButton;
+        m_resetButton->setToolTip(tr("Reset to Default"));
+        m_resetButton->setIcon(Icons::RESET.icon());
+        connect(m_resetButton.data(), &QAbstractButton::clicked,
+                this, &ArgumentsAspect::resetArguments);
+        containerLayout->addWidget(m_resetButton);
+        containerLayout->setAlignment(m_resetButton, Qt::AlignTop);
+    }
+
+    builder.addItems({m_labelText, container});
 }
 
 /*!
     \class ProjectExplorer::ExecutableAspect
+    \inmodule QtCreator
+
+    \brief The ExecutableAspect class provides a building block to provide an
+    executable for a RunConfiguration.
+
+    It combines a StringAspect that is typically updated automatically
+    by the build system's parsing results with an optional manual override.
 */
 
 ExecutableAspect::ExecutableAspect()
@@ -384,18 +507,27 @@ ExecutableAspect::ExecutableAspect()
     setExecutablePathStyle(HostOsInfo::hostOs());
     m_executable.setPlaceHolderText(tr("<unknown>"));
     m_executable.setLabelText(tr("Executable:"));
-    m_executable.setDisplayStyle(BaseStringAspect::LabelDisplay);
+    m_executable.setDisplayStyle(StringAspect::LabelDisplay);
 
-    connect(&m_executable, &BaseStringAspect::changed,
+    connect(&m_executable, &StringAspect::changed,
             this, &ExecutableAspect::changed);
 }
 
+/*!
+    \internal
+*/
 ExecutableAspect::~ExecutableAspect()
 {
     delete m_alternativeExecutable;
     m_alternativeExecutable = nullptr;
 }
 
+/*!
+   Sets the display style of the paths to the default used on \a osType,
+   backslashes on Windows, forward slashes elsewhere.
+
+   \sa Utils::StringAspect::setDisplayFilter()
+*/
 void ExecutableAspect::setExecutablePathStyle(OsType osType)
 {
     m_executable.setDisplayFilter([osType](const QString &pathName) {
@@ -403,6 +535,11 @@ void ExecutableAspect::setExecutablePathStyle(OsType osType)
     });
 }
 
+/*!
+   Sets the settings key for history completion to \a historyCompleterKey.
+
+   \sa Utils::PathChooser::setHistoryCompleter()
+*/
 void ExecutableAspect::setHistoryCompleter(const QString &historyCompleterKey)
 {
     m_executable.setHistoryCompleter(historyCompleterKey);
@@ -410,6 +547,11 @@ void ExecutableAspect::setHistoryCompleter(const QString &historyCompleterKey)
         m_alternativeExecutable->setHistoryCompleter(historyCompleterKey);
 }
 
+/*!
+   Sets the acceptable kind of path values to \a expectedKind.
+
+   \sa Utils::PathChooser::setExpectedKind()
+*/
 void ExecutableAspect::setExpectedKind(const PathChooser::Kind expectedKind)
 {
     m_executable.setExpectedKind(expectedKind);
@@ -417,6 +559,13 @@ void ExecutableAspect::setExpectedKind(const PathChooser::Kind expectedKind)
         m_alternativeExecutable->setExpectedKind(expectedKind);
 }
 
+/*!
+   Sets the environment in which paths will be searched when the expected kind
+   of paths is chosen as PathChooser::Command or PathChooser::ExistingCommand
+   to \a env.
+
+   \sa Utils::StringAspect::setEnvironment()
+*/
 void ExecutableAspect::setEnvironment(const Environment &env)
 {
     m_executable.setEnvironment(env);
@@ -424,24 +573,45 @@ void ExecutableAspect::setEnvironment(const Environment &env)
         m_alternativeExecutable->setEnvironment(env);
 }
 
-void ExecutableAspect::setDisplayStyle(BaseStringAspect::DisplayStyle style)
+/*!
+   Sets the display \a style for aspect.
+
+   \sa Utils::StringAspect::setDisplayStyle()
+*/
+void ExecutableAspect::setDisplayStyle(StringAspect::DisplayStyle style)
 {
     m_executable.setDisplayStyle(style);
 }
 
+/*!
+   Makes an auto-detected executable overridable by the user.
+
+   The \a overridingKey specifies the settings key for the user-provided executable,
+   the \a useOverridableKey the settings key for the fact that it
+   is actually overridden the user.
+
+   \sa Utils::StringAspect::makeCheckable()
+*/
 void ExecutableAspect::makeOverridable(const QString &overridingKey, const QString &useOverridableKey)
 {
     QTC_ASSERT(!m_alternativeExecutable, return);
-    m_alternativeExecutable = new BaseStringAspect;
-    m_alternativeExecutable->setDisplayStyle(BaseStringAspect::LineEditDisplay);
+    m_alternativeExecutable = new StringAspect;
+    m_alternativeExecutable->setDisplayStyle(StringAspect::LineEditDisplay);
     m_alternativeExecutable->setLabelText(tr("Alternate executable on device:"));
     m_alternativeExecutable->setSettingsKey(overridingKey);
-    m_alternativeExecutable->makeCheckable(BaseStringAspect::CheckBoxPlacement::Right,
+    m_alternativeExecutable->makeCheckable(StringAspect::CheckBoxPlacement::Right,
                                            tr("Use this command instead"), useOverridableKey);
-    connect(m_alternativeExecutable, &BaseStringAspect::changed,
+    connect(m_alternativeExecutable, &StringAspect::changed,
             this, &ExecutableAspect::changed);
 }
 
+/*!
+    Returns the path of the executable specified by this aspect. In case
+    the user selected a manual override this will be the value specified
+    by the user.
+
+    \sa makeOverridable()
+ */
 FilePath ExecutableAspect::executable() const
 {
     if (m_alternativeExecutable && m_alternativeExecutable->isChecked())
@@ -450,35 +620,59 @@ FilePath ExecutableAspect::executable() const
     return m_executable.filePath();
 }
 
+/*!
+    \reimp
+*/
 void ExecutableAspect::addToLayout(LayoutBuilder &builder)
 {
     m_executable.addToLayout(builder);
     if (m_alternativeExecutable)
-        m_alternativeExecutable->addToLayout(builder.startNewRow());
+        m_alternativeExecutable->addToLayout(builder.finishRow());
 }
 
+/*!
+    Sets the label text for the main chooser to
+    \a labelText.
+
+    \sa Utils::StringAspect::setLabelText()
+*/
 void ExecutableAspect::setLabelText(const QString &labelText)
 {
     m_executable.setLabelText(labelText);
 }
 
+/*!
+    Sets the place holder text for the main chooser to
+    \a placeHolderText.
+
+    \sa Utils::StringAspect::setPlaceHolderText()
+*/
 void ExecutableAspect::setPlaceHolderText(const QString &placeHolderText)
 {
     m_executable.setPlaceHolderText(placeHolderText);
 }
 
+/*!
+    Sets the value of the main chooser to \a executable.
+*/
 void ExecutableAspect::setExecutable(const FilePath &executable)
 {
    m_executable.setFilePath(executable);
    m_executable.setShowToolTipOnLabel(true);
 }
 
+/*!
+    Sets the settings key to \a key.
+*/
 void ExecutableAspect::setSettingsKey(const QString &key)
 {
-    ProjectConfigurationAspect::setSettingsKey(key);
+    BaseAspect::setSettingsKey(key);
     m_executable.setSettingsKey(key);
 }
 
+/*!
+  \reimp
+*/
 void ExecutableAspect::fromMap(const QVariantMap &map)
 {
     m_executable.fromMap(map);
@@ -486,6 +680,9 @@ void ExecutableAspect::fromMap(const QVariantMap &map)
         m_alternativeExecutable->fromMap(map);
 }
 
+/*!
+   \reimp
+*/
 void ExecutableAspect::toMap(QVariantMap &map) const
 {
     m_executable.toMap(map);
@@ -496,6 +693,14 @@ void ExecutableAspect::toMap(QVariantMap &map) const
 
 /*!
     \class ProjectExplorer::UseLibraryPathsAspect
+    \inmodule QtCreator
+
+    \brief The UseLibraryPathsAspect class lets a user specify whether build
+    library search paths should be added to the relevant environment
+    variables.
+
+    This modifies DYLD_LIBRARY_PATH and DYLD_FRAMEWORK_PATH on Mac, PATH
+    on Windows and LD_LIBRARY_PATH everywhere else.
 */
 
 UseLibraryPathsAspect::UseLibraryPathsAspect()
@@ -514,8 +719,13 @@ UseLibraryPathsAspect::UseLibraryPathsAspect()
     setValue(ProjectExplorerPlugin::projectExplorerSettings().addLibraryPathsToRunEnv);
 }
 
+
 /*!
     \class ProjectExplorer::UseDyldSuffixAspect
+    \inmodule QtCreator
+
+    \brief The UseDyldSuffixAspect class lets a user specify whether the
+    DYLD_IMAGE_SUFFIX environment variable should be used on Mac.
 */
 
 UseDyldSuffixAspect::UseDyldSuffixAspect()
@@ -526,4 +736,19 @@ UseDyldSuffixAspect::UseDyldSuffixAspect()
              LabelPlacement::AtCheckBox);
 }
 
+/*!
+    \class ProjectExplorer::RunAsRootAspect
+    \inmodule QtCreator
+
+    \brief The RunAsRootAspect class lets a user specify whether the
+    application should run with root permissions.
+*/
+
+RunAsRootAspect::RunAsRootAspect()
+{
+    setId("RunAsRoot");
+    setSettingsKey("RunConfiguration.RunAsRoot");
+    setLabel(tr("Run as root user"), LabelPlacement::AtCheckBox);
+}
+
 } // namespace ProjectExplorer
