@@ -30,51 +30,40 @@
 
 #include <QEasingCurve>
 #include <QPainter>
-#include <QPainterPath>
 
 #include <algorithm>
 #include <cmath>
 #include <sstream>
 
-namespace DesignTools {
+namespace QmlDesigner {
 
 CurveItem::CurveItem(QGraphicsItem *parent)
-    : QGraphicsObject(parent)
+    : CurveEditorItem(parent)
     , m_id(0)
     , m_style()
-    , m_type(ValueType::Undefined)
+    , m_type(PropertyTreeItem::ValueType::Undefined)
     , m_component(PropertyTreeItem::Component::Generic)
     , m_transform()
     , m_keyframes()
-    , m_underMouse(false)
     , m_itemDirty(false)
 {}
 
 CurveItem::CurveItem(unsigned int id, const AnimationCurve &curve, QGraphicsItem *parent)
-    : QGraphicsObject(parent)
+    : CurveEditorItem(parent)
     , m_id(id)
     , m_style()
-    , m_type(ValueType::Undefined)
+    , m_type(PropertyTreeItem::ValueType::Undefined)
     , m_component(PropertyTreeItem::Component::Generic)
     , m_transform()
     , m_keyframes()
-    , m_underMouse(false)
     , m_itemDirty(false)
 {
-    setAcceptHoverEvents(true);
-
     setFlag(QGraphicsItem::ItemIsMovable, false);
-
     setCurve(curve);
 }
 
 CurveItem::~CurveItem() {}
 
-bool CurveItem::isUnderMouse() const
-{
-    return m_underMouse;
-}
-
 int CurveItem::type() const
 {
     return Type;
@@ -98,6 +87,16 @@ QRectF CurveItem::boundingRect() const
     for (auto *item : m_keyframes)
         bbox(bounds, item->keyframe());
 
+    if (auto *s = qobject_cast<GraphicsScene *>(scene())) {
+        bounds.setLeft(s->animationRangeMin());
+        bounds.setRight(s->animationRangeMax());
+    }
+
+    if (qFuzzyCompare(bounds.height(), 0.0)) {
+        auto tmp = CurveEditorStyle::defaultValueRange() / 2.0;
+        bounds.adjust(0.0, -tmp, 0.0, tmp);
+    }
+
     return m_transform.mapRect(bounds);
 }
 
@@ -128,9 +127,13 @@ void CurveItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidg
             if (segment.interpolation() == Keyframe::Interpolation::Easing) {
                 pen.setColor(m_style.easingCurveColor);
             } else {
-                if (m_underMouse)
-                    pen.setColor(Qt::red);
-                else if (hasSelection())
+                if (locked())
+                    pen.setColor(m_style.lockedColor);
+                else if (!segment.isLegal())
+                    pen.setColor(m_style.errorColor);
+                else if (isUnderMouse())
+                    pen.setColor(m_style.hoverColor);
+                else if (hasSelectedKeyframe())
                     pen.setColor(m_style.selectionColor);
                 else
                     pen.setColor(m_style.color);
@@ -142,12 +145,38 @@ void CurveItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidg
     }
 }
 
+void CurveItem::lockedCallback()
+{
+    for (auto frame : qAsConst(m_keyframes))
+        frame->setLocked(locked());
+
+    setHandleVisibility(!locked());
+}
+
 bool CurveItem::isDirty() const
 {
     return m_itemDirty;
 }
 
-bool CurveItem::hasSelection() const
+bool CurveItem::hasActiveKeyframe() const
+{
+    for (auto *frame : m_keyframes) {
+        if (frame->activated())
+            return true;
+    }
+    return false;
+}
+
+bool CurveItem::hasActiveHandle() const
+{
+    for (auto *frame : m_keyframes) {
+        if (frame->hasActiveHandle())
+            return true;
+    }
+    return false;
+}
+
+bool CurveItem::hasSelectedKeyframe() const
 {
     for (auto *frame : m_keyframes) {
         if (frame->selected())
@@ -156,12 +185,47 @@ bool CurveItem::hasSelection() const
     return false;
 }
 
+bool CurveItem::hasEditableSegment(double time) const
+{
+    return curve().segment(time).interpolation() != Keyframe::Interpolation::Easing;
+}
+
+bool CurveItem::isFirst(const KeyframeItem *key) const
+{
+    if (m_keyframes.empty())
+        return false;
+
+    return m_keyframes.first() == key;
+}
+
+bool CurveItem::isLast(const KeyframeItem *key) const
+{
+    if (m_keyframes.empty())
+        return false;
+
+    return m_keyframes.last() == key;
+}
+
+int CurveItem::indexOf(const KeyframeItem *key) const
+{
+    if (m_keyframes.empty())
+        return -1;
+
+    int out = 0;
+    for (auto &&el : m_keyframes) {
+        if (el == key)
+            return out;
+        out++;
+    }
+    return -1;
+}
+
 unsigned int CurveItem::id() const
 {
     return m_id;
 }
 
-ValueType CurveItem::valueType() const
+PropertyTreeItem::ValueType CurveItem::valueType() const
 {
     return m_type;
 }
@@ -204,7 +268,7 @@ std::vector<AnimationCurve> CurveItem::curves() const
 
     std::vector<Keyframe> tmp;
 
-    for (size_t i = 0; i < m_keyframes.size(); ++i) {
+    for (int i = 0; i < m_keyframes.size(); ++i) {
         KeyframeItem *item = m_keyframes[i];
 
         Keyframe current = item->keyframe();
@@ -234,6 +298,49 @@ std::vector<AnimationCurve> CurveItem::curves() const
     return out;
 }
 
+QVector<KeyframeItem *> CurveItem::keyframes() const
+{
+    return m_keyframes;
+}
+
+QVector<KeyframeItem *> CurveItem::selectedKeyframes() const
+{
+    QVector<KeyframeItem *> out;
+    for (auto *frame : m_keyframes) {
+        if (frame->selected())
+            out.push_back(frame);
+    }
+    return out;
+}
+
+QVector<HandleItem *> CurveItem::handles() const
+{
+    QVector<HandleItem *> out;
+    for (auto *frame : m_keyframes) {
+        if (auto *left = frame->leftHandle())
+            out.push_back(left);
+        if (auto *right = frame->rightHandle())
+            out.push_back(right);
+    }
+    return out;
+}
+
+CurveSegment CurveItem::segment(const KeyframeItem *keyframe, HandleItem::Slot slot) const
+{
+    auto finder = [keyframe](KeyframeItem *item) { return item == keyframe; };
+    const auto iter = std::find_if(m_keyframes.cbegin(), m_keyframes.cend(), finder);
+    if (iter == m_keyframes.cend())
+        return CurveSegment();
+
+    int index = static_cast<int>(std::distance(m_keyframes.cbegin(), iter));
+    if (slot == HandleItem::Slot::Left && index > 0)
+        return CurveSegment(m_keyframes[index - 1]->keyframe(), keyframe->keyframe());
+    else if (slot == HandleItem::Slot::Right && index < (m_keyframes.size() - 1))
+        return CurveSegment(keyframe->keyframe(), m_keyframes[index + 1]->keyframe());
+
+    return CurveSegment();
+}
+
 void CurveItem::restore()
 {
     if (m_keyframes.empty())
@@ -245,17 +352,24 @@ void CurveItem::restore()
     std::sort(m_keyframes.begin(), m_keyframes.end(), byTime);
 
     KeyframeItem *prevItem = m_keyframes[0];
-    for (size_t i = 1; i < m_keyframes.size(); ++i) {
+
+    if (prevItem->hasLeftHandle())
+        prevItem->setLeftHandle(QPointF());
+
+    for (int i = 1; i < m_keyframes.size(); ++i) {
         KeyframeItem *currItem = m_keyframes[i];
 
-        Keyframe prev = prevItem->keyframe();
-        Keyframe curr = currItem->keyframe();
-        CurveSegment segment(prev, curr);
+        bool left = prevItem->hasRightHandle();
+        bool right = currItem->hasLeftHandle();
+        if (left != right) {
+            if (left)
+                prevItem->setRightHandle(QPointF());
 
-        prevItem->setRightHandle(segment.left().rightHandle());
+            if (right)
+                currItem->setLeftHandle(QPointF());
+        }
+        CurveSegment segment(prevItem->keyframe(), currItem->keyframe());
         currItem->setInterpolation(segment.interpolation());
-        currItem->setLeftHandle(segment.right().leftHandle());
-
         prevItem = currItem;
     }
 }
@@ -267,11 +381,11 @@ void CurveItem::setDirty(bool dirty)
 
 void CurveItem::setHandleVisibility(bool visible)
 {
-    for (auto frame : m_keyframes)
+    for (auto frame : qAsConst(m_keyframes))
         frame->setHandleVisibility(visible);
 }
 
-void CurveItem::setValueType(ValueType type)
+void CurveItem::setValueType(PropertyTreeItem::ValueType type)
 {
     m_type = type;
 }
@@ -285,11 +399,14 @@ void CurveItem::setCurve(const AnimationCurve &curve)
 {
     freeClear(m_keyframes);
 
-    for (auto frame : curve.keyframes()) {
+    for (const auto &frame : curve.keyframes()) {
         auto *item = new KeyframeItem(frame, this);
+        item->setLocked(locked());
         item->setComponentTransform(m_transform);
         m_keyframes.push_back(item);
         QObject::connect(item, &KeyframeItem::redrawCurve, this, &CurveItem::emitCurveChanged);
+        QObject::connect(item, &KeyframeItem::keyframeMoved, this, &CurveItem::keyframeMoved);
+        QObject::connect(item, &KeyframeItem::handleMoved, this, &CurveItem::handleMoved);
     }
 
     emitCurveChanged();
@@ -299,7 +416,7 @@ QRectF CurveItem::setComponentTransform(const QTransform &transform)
 {
     prepareGeometryChange();
     m_transform = transform;
-    for (auto frame : m_keyframes)
+    for (auto frame : qAsConst(m_keyframes))
         frame->setComponentTransform(transform);
 
     return boundingRect();
@@ -309,7 +426,7 @@ void CurveItem::setStyle(const CurveEditorStyle &style)
 {
     m_style = style.curveStyle;
 
-    for (auto *frame : m_keyframes)
+    for (auto *frame : qAsConst(m_keyframes))
         frame->setStyle(style);
 }
 
@@ -319,7 +436,7 @@ void CurveItem::setInterpolation(Keyframe::Interpolation interpolation)
         return;
 
     KeyframeItem *prevItem = m_keyframes[0];
-    for (size_t i = 1; i < m_keyframes.size(); ++i) {
+    for (int i = 1; i < m_keyframes.size(); ++i) {
         KeyframeItem *currItem = m_keyframes[i];
         if (currItem->selected()) {
             Keyframe prev = prevItem->keyframe();
@@ -337,26 +454,31 @@ void CurveItem::setInterpolation(Keyframe::Interpolation interpolation)
     emit curveChanged(id(), curve());
 }
 
+void CurveItem::toggleUnified()
+{
+    if (m_keyframes.empty())
+        return;
+
+    for (auto *frame : qAsConst(m_keyframes)) {
+        if (frame->selected())
+            frame->toggleUnified();
+    }
+    emit curveChanged(id(), curve());
+}
+
 void CurveItem::connect(GraphicsScene *scene)
 {
     QObject::connect(this, &CurveItem::curveChanged, scene, &GraphicsScene::curveChanged);
 
-    for (auto *frame : m_keyframes) {
-        QObject::connect(frame, &KeyframeItem::keyframeMoved, scene, &GraphicsScene::keyframeMoved);
-        QObject::connect(frame, &KeyframeItem::handleMoved, scene, &GraphicsScene::handleMoved);
-    }
-}
-
-void CurveItem::setIsUnderMouse(bool under)
-{
-    if (under != m_underMouse) {
-        m_underMouse = under;
-        update();
-    }
+    QObject::connect(this, &CurveItem::keyframeMoved, scene, &GraphicsScene::keyframeMoved);
+    QObject::connect(this, &CurveItem::handleMoved, scene, &GraphicsScene::handleMoved);
 }
 
 void CurveItem::insertKeyframeByTime(double time)
 {
+    if (locked())
+        return;
+
     AnimationCurve acurve = curve();
     acurve.insert(time);
     setCurve(acurve);
@@ -375,6 +497,8 @@ void CurveItem::deleteSelectedKeyframes()
     auto isNullptr = [](KeyframeItem *frame) { return frame == nullptr; };
     auto iter = std::remove_if(m_keyframes.begin(), m_keyframes.end(), isNullptr);
     m_keyframes.erase(iter, m_keyframes.end());
+    restore();
+
     emitCurveChanged();
 }
 
@@ -384,4 +508,4 @@ void CurveItem::emitCurveChanged()
     update();
 }
 
-} // End namespace DesignTools.
+} // End namespace QmlDesigner.
