@@ -33,15 +33,16 @@
 namespace QmlDesigner {
 
 class ItemLibraryWidget;
-class ImportManagerView;
+class ImageCacheData;
+class AsynchronousImageCache;
 
 class ItemLibraryView : public AbstractView
 {
     Q_OBJECT
 
 public:
-    ItemLibraryView(QObject* parent = 0);
-    ~ItemLibraryView();
+    ItemLibraryView(QObject* parent = nullptr);
+    ~ItemLibraryView() override;
 
     bool hasWidget() const override;
     WidgetInfo widgetInfo() override;
@@ -50,15 +51,26 @@ public:
     void modelAttached(Model *model) override;
     void modelAboutToBeDetached(Model *model) override;
     void importsChanged(const QList<Import> &addedImports, const QList<Import> &removedImports) override;
+    void possibleImportsChanged(const QList<Import> &possibleImports) override;
+    void usedImportsChanged(const QList<Import> &usedImports) override;
+    void documentMessagesChanged(const QList<DocumentMessage> &errors, const QList<DocumentMessage> &warnings) override;
+    void updateImport3DSupport(const QVariantMap &supportMap) override;
+    void customNotification(const AbstractView *view, const QString &identifier,
+                            const QList<ModelNode> &nodeList, const QList<QVariant> &data) override;
 
     void setResourcePath(const QString &resourcePath);
 
+    AsynchronousImageCache &imageCache();
+
 protected:
     void updateImports();
 
 private:
+    std::unique_ptr<ImageCacheData> m_imageCacheData;
     QPointer<ItemLibraryWidget> m_widget;
-    ImportManagerView *m_importManagerView;
+    bool m_hasErrors = false;
+    QVariantMap m_importableExtensions3DMap;
+    QVariantMap m_importOptions3DMap;
 };
 
 }
