@@ -25,6 +25,7 @@
 
 #include "subcomponentmanager.h"
 
+#include <itemlibraryimport.h>
 #include <qmldesignerconstants.h>
 
 #include "model.h"
@@ -35,6 +36,7 @@
 #include <coreplugin/messagebox.h>
 
 #include <QDir>
+#include <QDirIterator>
 #include <QMessageBox>
 #include <QUrl>
 
@@ -67,10 +69,10 @@ SubComponentManager::SubComponentManager(Model *model, QObject *parent)
             this, [this](const QString &path) { parseDirectory(path); });
 }
 
-void SubComponentManager::addImport(int pos, const Import &import)
+void SubComponentManager::addImport(const Import &import, int index)
 {
     if (debug)
-        qDebug() << Q_FUNC_INFO << pos << import.file().toUtf8();
+        qDebug() << Q_FUNC_INFO << index << import.file().toUtf8();
 
     if (import.isFileImport()) {
         QFileInfo dirInfo = QFileInfo(m_filePath.resolved(import.file()).toLocalFile());
@@ -85,8 +87,7 @@ void SubComponentManager::addImport(int pos, const Import &import)
         url.replace(QLatin1Char('.'), QLatin1Char('/'));
 
         foreach (const QString &path, importPaths()) {
-            url  = path + QLatin1Char('/') + url;
-            QFileInfo dirInfo = QFileInfo(url);
+            QFileInfo dirInfo = QFileInfo(path + QLatin1Char('/') + url);
             if (dirInfo.exists() && dirInfo.isDir()) {
                 const QString canonicalDirPath = dirInfo.canonicalFilePath();
                 m_watcher.addPath(canonicalDirPath);
@@ -96,12 +97,15 @@ void SubComponentManager::addImport(int pos, const Import &import)
         // TODO: QDeclarativeDomImport::Library
     }
 
-    m_imports.insert(pos, import);
+    if (index == -1)
+        m_imports.append(import);
+    else
+        m_imports.insert(index, import);
 }
 
-void SubComponentManager::removeImport(int pos)
+void SubComponentManager::removeImport(int index)
 {
-    const Import import = m_imports.takeAt(pos);
+    const Import import = m_imports.takeAt(index);
 
     if (import.isFileImport()) {
         const QFileInfo dirInfo = QFileInfo(m_filePath.resolved(import.file()).toLocalFile());
@@ -126,14 +130,19 @@ void SubComponentManager::parseDirectories()
     if (!m_filePath.isEmpty()) {
         const QString file = m_filePath.toLocalFile();
         QFileInfo dirInfo = QFileInfo(QFileInfo(file).path());
+        const QString canonicalPath = dirInfo.canonicalFilePath();
         if (dirInfo.exists() && dirInfo.isDir())
-            parseDirectory(dirInfo.canonicalFilePath());
+            parseDirectory(canonicalPath);
 
         foreach (const QString &subDir, QDir(QFileInfo(file).path()).entryList(QDir::Dirs | QDir::NoDot | QDir::NoDotDot)) {
-            parseDirectory(dirInfo.canonicalFilePath() + QLatin1String("/") + subDir, true, subDir.toUtf8());
+            parseDirectory(canonicalPath + QLatin1Char('/') + subDir, true, subDir.toUtf8());
         }
     }
 
+    const QStringList assetPaths = quick3DAssetPaths();
+    for (const auto &assetPath : assetPaths)
+        parseDirectory(assetPath);
+
     foreach (const Import &import, m_imports) {
         if (import.isFileImport()) {
             QFileInfo dirInfo = QFileInfo(m_filePath.resolved(import.file()).toLocalFile());
@@ -152,7 +161,7 @@ void SubComponentManager::parseDirectories()
                     parseDirectory(dirInfo.canonicalFilePath(), false);
                 }
 
-                QString fullUrlVersion = path + QLatin1Char('/') + url + QLatin1Char('.') + import.version().split(".").constFirst();
+                QString fullUrlVersion = path + QLatin1Char('/') + url + QLatin1Char('.') + import.version().split('.').constFirst();
                 dirInfo = QFileInfo(fullUrlVersion);
 
                 if (dirInfo.exists() && dirInfo.isDir()) {
@@ -164,11 +173,16 @@ void SubComponentManager::parseDirectories()
     }
 }
 
-void SubComponentManager::parseDirectory(const QString &canonicalDirPath, bool addToLibrary, const TypeName& qualification)
+void SubComponentManager::parseDirectory(const QString &canonicalDirPath, bool addToLibrary, const TypeName &qualification)
 {
     if (!model() || !model()->rewriterView())
         return;
 
+    if (canonicalDirPath.endsWith(QLatin1String(Constants::QUICK_3D_ASSETS_FOLDER))) {
+        parseQuick3DAssetsDir(canonicalDirPath);
+        return;
+    }
+
     QDir designerDir(canonicalDirPath + QLatin1String(Constants::QML_DESIGNER_SUBFOLDER));
     if (designerDir.exists()) {
         QStringList filter;
@@ -258,7 +272,7 @@ void SubComponentManager::parseDirectory(const QString &canonicalDirPath, bool a
     }
 }
 
-void SubComponentManager::parseFile(const QString &canonicalFilePath, bool addToLibrary, const QString&  qualification)
+void SubComponentManager::parseFile(const QString &canonicalFilePath, bool addToLibrary, const QString &qualification)
 {
     if (debug)
         qDebug() << Q_FUNC_INFO << canonicalFilePath;
@@ -267,11 +281,12 @@ void SubComponentManager::parseFile(const QString &canonicalFilePath, bool addTo
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
         return;
 
-    QString dir = QFileInfo(canonicalFilePath).path();
+    const QFileInfo fi(canonicalFilePath);
+    const QString dir = fi.path();
     foreach (const QString &qualifier, m_dirToQualifier.values(dir)) {
-        registerQmlFile(canonicalFilePath, qualifier, addToLibrary);
+        registerQmlFile(fi, qualifier, addToLibrary);
     }
-    registerQmlFile(canonicalFilePath, qualification, addToLibrary);
+    registerQmlFile(fi, qualification, addToLibrary);
 }
 
 void SubComponentManager::parseFile(const QString &canonicalFilePath)
@@ -301,7 +316,7 @@ void SubComponentManager::unregisterQmlFile(const QFileInfo &fileInfo, const QSt
 
 
 void SubComponentManager::registerQmlFile(const QFileInfo &fileInfo, const QString &qualifier,
-                                                 bool addToLibrary)
+                                          bool addToLibrary)
 {
     if (!model())
         return;
@@ -325,7 +340,10 @@ void SubComponentManager::registerQmlFile(const QFileInfo &fileInfo, const QStri
         ItemLibraryEntry itemLibraryEntry;
         itemLibraryEntry.setType(componentName.toUtf8());
         itemLibraryEntry.setName(baseComponentName);
-        itemLibraryEntry.setCategory(QLatin1String("My QML Components"));
+#ifndef QMLDESIGNER_TEST
+        itemLibraryEntry.setCategory(ItemLibraryImport::userComponentsTitle());
+#endif
+        itemLibraryEntry.setCustomComponentSource(fileInfo.absoluteFilePath());
         if (!qualifier.isEmpty()) {
             itemLibraryEntry.setRequiredImport(fixedQualifier);
         }
@@ -348,6 +366,87 @@ QStringList SubComponentManager::importPaths() const
     return QStringList();
 }
 
+void SubComponentManager::parseQuick3DAssetsDir(const QString &quick3DAssetsPath)
+{
+    QDir quick3DAssetsDir(quick3DAssetsPath);
+    QStringList assets = quick3DAssetsDir.entryList(QDir::Dirs | QDir::NoDot | QDir::NoDotDot);
+    for (QString &asset : assets)
+        asset.prepend(QString(Constants::QUICK_3D_ASSETS_FOLDER).mid(1) + '.');
+
+    // Create item library entries for Quick3D assets that are imported by document
+    for (auto &import : qAsConst(m_imports)) {
+        if (import.isLibraryImport() && assets.contains(import.url())) {
+            assets.removeOne(import.url());
+            parseQuick3DAssetsItem(import.url(), quick3DAssetsPath);
+        }
+    }
+}
+
+// parse one asset folder under Quick3DAssets
+void SubComponentManager::parseQuick3DAssetsItem(const QString &importUrl, const QString &quick3DAssetsPath)
+{
+    QString assetPath;
+    if (!quick3DAssetsPath.isEmpty()) {
+        assetPath = QDir(quick3DAssetsPath).filePath(importUrl.split('.').last());
+    } else {
+        // search for quick3DAssetsPath if not provided
+        const auto assetPaths = quick3DAssetPaths();
+        for (const auto &quick3DAssetPath : assetPaths) {
+            QString path = QDir(quick3DAssetPath).filePath(importUrl.split('.').last());
+            if (QFileInfo::exists(path)) {
+                assetPath = path;
+                break;
+            }
+        }
+    }
+
+    const QString defaultIconPath = QStringLiteral(":/ItemLibrary/images/item-3D_model-icon.png");
+    QDirIterator qmlIt(assetPath, {"*.qml"}, QDir::Files);
+    while (qmlIt.hasNext()) {
+        qmlIt.next();
+        const QString name = qmlIt.fileInfo().baseName();
+        const QString type = importUrl + '.' + name;
+        // For now we assume version is always 1.0 as that's what importer hardcodes
+        ItemLibraryEntry itemLibraryEntry;
+        itemLibraryEntry.setType(type.toUtf8(), 1, 0);
+        itemLibraryEntry.setName(name);
+        itemLibraryEntry.setCategory(tr("My Quick3D Components"));
+        itemLibraryEntry.setRequiredImport(importUrl);
+        QString iconPath = qmlIt.fileInfo().absolutePath() + '/'
+                + Constants::QUICK_3D_ASSET_ICON_DIR + '/' + name
+                + Constants::QUICK_3D_ASSET_LIBRARY_ICON_SUFFIX;
+        if (!QFileInfo::exists(iconPath))
+            iconPath = defaultIconPath;
+        itemLibraryEntry.setLibraryEntryIconPath(iconPath);
+        itemLibraryEntry.setTypeIcon(QIcon(iconPath));
+
+        // load hints file if exists
+        QFile hintsFile(qmlIt.fileInfo().absolutePath() + '/' + name + ".hints");
+        if (hintsFile.exists() && hintsFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
+            QTextStream in(&hintsFile);
+            QHash<QString, QString> hints;
+            while (!in.atEnd()) {
+                QStringList hint = in.readLine().split(':');
+                hints.insert(hint[0].trimmed(), hint[1].trimmed());
+            }
+            itemLibraryEntry.addHints(hints);
+        }
+
+        model()->metaInfo().itemLibraryInfo()->addEntries({itemLibraryEntry}, true);
+    }
+}
+
+QStringList SubComponentManager::quick3DAssetPaths() const
+{
+    const auto impPaths = importPaths();
+    QStringList retPaths;
+    for (const auto &impPath : impPaths) {
+        const QString assetPath = impPath + QLatin1String(Constants::QUICK_3D_ASSETS_FOLDER);
+        if (QFileInfo::exists(assetPath))
+            retPaths << assetPath;
+    }
+    return retPaths;
+}
 
 /*!
   \class SubComponentManager
@@ -392,10 +491,18 @@ void SubComponentManager::update(const QUrl &filePath, const QList<Import> &impo
             m_dirToQualifier.remove(oldDir.canonicalFilePath(), QString());
             if (!m_dirToQualifier.contains(oldDir.canonicalFilePath()))
                 m_watcher.removePath(oldDir.filePath());
+
+            // Remove old watched asset paths
+            const QStringList watchPaths = m_watcher.directories();
+            const QString &quick3DAssetFolder = QLatin1String(Constants::QUICK_3D_ASSETS_FOLDER);
+            for (const auto &watchPath : watchPaths) {
+                if (watchPath.endsWith(quick3DAssetFolder))
+                    m_watcher.removePath(watchPath);
+            }
         }
 
         if (!newDir.filePath().isEmpty())
-            m_dirToQualifier.insertMulti(newDir.canonicalFilePath(), QString());
+            m_dirToQualifier.insert(newDir.canonicalFilePath(), QString());
     }
 
     //
@@ -414,13 +521,61 @@ void SubComponentManager::update(const QUrl &filePath, const QList<Import> &impo
         removeImport(ii);
 
     for (int ii = i; ii < imports.size(); ++ii) {
-        addImport(ii, imports.at(ii));
+        addImport(imports.at(ii), ii);
     }
 
-    m_watcher.addPath(newDir.absoluteFilePath());
+    const QString newPath = newDir.absoluteFilePath();
+    m_watcher.addPath(newPath);
+
+    // Watch existing asset paths, including a global ones if they exist
+    const auto assetPaths = quick3DAssetPaths();
+    for (const auto &assetPath : assetPaths)
+        m_watcher.addPath(assetPath);
 
     parseDirectories();
 }
 
+void SubComponentManager::updateImport(const Import &import)
+{
+    addImport(import);
+
+    if (import.isFileImport()) {
+        QFileInfo dirInfo = QFileInfo(m_filePath.resolved(import.file()).toLocalFile());
+        if (dirInfo.exists() && dirInfo.isDir())
+            parseDirectory(dirInfo.canonicalFilePath(), true, dirInfo.baseName().toUtf8());
+    } else {
+        QString url = import.url();
+
+        if (url.startsWith(QString(Constants::QUICK_3D_ASSETS_FOLDER).mid(1))) {
+            parseQuick3DAssetsItem(import.url());
+            return;
+        }
+        url.replace('.', '/');
+        QFileInfo dirInfo = QFileInfo(url);
+        const QStringList importPathList = importPaths();
+        bool parsed = false;
+        for (const QString &path : importPathList) {
+            QString fullUrl  = path + '/' + url;
+            dirInfo = QFileInfo(fullUrl);
+
+            if (dirInfo.exists() && dirInfo.isDir()) {
+                parseDirectory(dirInfo.canonicalFilePath(), false);
+                parsed = true;
+            }
+
+            QString fullUrlVersion = path + '/' + url + '.' + import.version().split('.').constFirst();
+            dirInfo = QFileInfo(fullUrlVersion);
+
+            if (dirInfo.exists() && dirInfo.isDir()) {
+                parseDirectory(dirInfo.canonicalFilePath(), false);
+                parsed = true;
+            }
+
+            if (parsed)
+                break;
+        }
+    }
+}
+
 } // namespace QmlDesigner
 
